/*
 *     nanoMir
 *
 *     "le.c"           Line editor
 *
 */

#include "nm.h"
#include "wd.h"
#include "cm.h"
#include "cmn.h"
#include "ud.h"
#include "le.h"
#include "te.h"
#include "ac.h"
#include "kb.h"
#include "cp.h"
#include "er.h"
#include "tx.h"

#include "keytable.c"

#ifdef X11
#include "xi.h"
#endif

#ifdef  VERSION_2
bool Lrins = TRUE;
#else
bool Lrins;
#endif /* VERSION_2 */
bool Lredit;
bool Lchange;
bool Lwp;
wnd *Lwnd;
txt *Ltxt;
large Ly;
small Lx;
small Lleng;
small Lxlm, Lxle, Lxrm, Lxre;
tchar Lattr;
tchar *Lebuf;
char *tabuf;

static tchar *ccbuf;
static small cclen;
static bool ccempt;
static small ccrov;

static tchar lchar;
static small lxl;

local void llchar (void), lcdword (void), leic2 (void), ledc2 (void),
leccase (char *), lekbwait (void);
local small leleng (void);
local bool lcword (void);
local void lecword1 ();
local tchar *lefill (tchar *, small, tchar *);



void lechar (void), ledeol (void), ledbgol (void), leic (void),
ledc (void), lebs (void), ledword (void), ledlword (void),
lecentr (void), lespchar (void), leescchr (void), lehfchar (void);
void leccup (void), leccdwn (void), leccrus (void), lecclat (void),
lecattr (void);
void lanone (void), lalight (void), laitalic (void), launderl (void),
labold (void), lasuper (void), lasubstr (void), lainvert (void);
void leright (void), leleft (void), lebeg (void), leend (void),
letab (void), leltab (void), lenword (void), lepword (void);
void lecchar (void), lecdchar (void), lecword (void), lecdword (void),
lepchar (void), leccchr (void);
void lerins (void), lerrep (void);

#define FSTLECOM 0400
comdesc lecmds[] = {
  lechar, /*400 */ CA_EXT | CA_CHANGE | CA_NEND,
  leright, /*401 */ CA_NEND,
  leleft, /*402 */ CA_NBEG,
  ledeol, /*403 */ CA_CHANGE,
  ledbgol, /*404 */ CA_CHANGE,
  lebeg, /*405 */ CA_RPT,
  leend, /*406 */ CA_RPT,
  leic, /*407 */ CA_CHANGE | CA_NEND,
  ledc, /*410 */ CA_CHANGE | CA_NEND,
  lebs, /*411 */ CA_CHANGE | CA_NBEG,
  letab, /*412 */ CA_NEND,
  leltab, /*413 */ CA_NBEG,
  lenword, /*414 */ CA_NEND,
  lepword, /*415 */ CA_NBEG,
  lecchar, /*416 */ CA_NEND,
  lecdchar, /*417 */ CA_CHANGE | CA_NEND,
  lepchar, /*420 */ CA_EXT | CA_CHANGE | CA_NEND,
  leccchr, /*421 */ 0,
  lecword, /*422 */ CA_NEND,
  lecdword, /*423 */ CA_CHANGE | CA_NEND,
  ledword, /*424 */ CA_CHANGE | CA_NEND,
  ledlword, /*425 */ CA_CHANGE | CA_NBEG,
  lecentr, /*426 */ CA_CHANGE | CA_RPT,
  lerins, /*427 */ 0,
  lerrep, /*430 */ 0,
  leccup, /*431 */ CA_CHANGE | CA_NEND,
  leccdwn, /*432 */ CA_CHANGE | CA_NEND,
  leccrus, /*433 */ CA_CHANGE | CA_NEND,
  lecclat, /*434 */ CA_CHANGE | CA_NEND,
  lecattr, /*435 */ CA_CHANGE | CA_WP | CA_NEND,
  lespchar, /*436 */ CA_CHANGE | CA_RPT | CA_EXT | CA_NEND,
  leescchr, /*437 */ CA_CHANGE | CA_EXT | CA_NEND,	/* ESC (033 ) */
  lehfchar, /*440 */ CA_CHANGE | CA_EXT | CA_NEND,	/*          */
  lanone, /*441 */ CA_WP,
  lalight, /*442 */ CA_WP,
  laitalic, /*443 */ CA_WP,
  launderl, /*444 */ CA_WP,
  labold, /*445 */ CA_WP,
  lasuper, /*446 */ CA_WP,
  lasubstr, /*447 */ CA_WP,
  lainvert, /*450 */ CA_WP,
  leundo, /*451 */ CA_CHANGE,
  leunundo, /*452 */ CA_CHANGE,
  lesundo, /*453 */ CA_CHANGE,
  lesunundo, /*454 */ CA_CHANGE,
};

#define LSTLECOM 0454

void
leInit ()
{
  int i;
  int tabsize = (TX_TabSize ? TX_TabSize : 8);
  int tabpos;

  Lebuf = (tchar *) GetMain (MAXLPAC * sizeof (tchar));
  ccbuf = (tchar *) GetMain (MAXLPAC * sizeof (tchar));
  tabuf = GetMain (MAXLPAC);
  cclen = ccempt = ccrov = 0;
  for (i = 0, tabpos = 0; i < MAXLPAC; i++)
    /* tabuf[i] = i & 7 ? 0 : TABSTOP; */
    if (i == tabpos)
      {
	tabuf[i] = TABSTOP;
	tabpos += tabsize;
      }
    else
      tabuf[i] = 0;
  Lwnd = NIL;
}

comdesc *
Ldecode ()
{
  register kchar code;
  if ((code = KbCode) < 0400)
    return (&lecmds[0]);
  if ((code -= FSTLECOM) >= 0 && code <= (LSTLECOM - FSTLECOM))
    return (lecmds + code);
  return (NIL);
}

#define erret(  code ) { x=code; goto ret;  }
#define erret2( code ) { x=code; goto ret2; }

small
LExec (small typ)
{
  register comdesc *cp;
  register small attr;
  register small x;
  register jmp_buf *nextexc;
  jmp_buf leenv;
  large r;			/* Can't be register due to setjmp */
  register void (*kboldhook) ();
#ifdef NO_MARGIN_BELL
  small IsMargin = 0;
#endif

  nextexc = excptr;
  excptr = (jmp_buf *) leenv;
  kboldhook = KbWaitIn;
  if (x = setjmp (leenv))
    goto ret2;
  KbWaitIn = lekbwait;

  if (typ & RQ_EXEC)
    goto noinp;
loop:
  KbGet ();
noinp:
  if ((cp = Ldecode ()) == NIL)
    erret (E_NOCOM);
  attr = cp->cattr;
  r = attr & CA_RPT ? 1 : KbCount;
  if (attr & CA_WP && !Lwp)
    erret (E_ATTR);
  if (attr & CA_CHANGE)
    {
      if (!Lredit)
	erret (E_CHANGE);
      UdMark = TRUE;
    }
  for (; r; r--)
    {
      x = Lx;
#ifdef NO_MARGIN_BELL
      if (attr & CA_NEND && x >= Lxrm)
	{
	  IsMargin = 1;
	  erret2 (E_MOVEND);
	}
      if (attr & CA_NBEG && x <= Lxlm)
	{
	  IsMargin = 1;
	  erret2 (E_MOVBEG);
	}
#else
      if (attr & CA_NEND && x >= Lxrm)
	erret2 (E_MOVEND);
      if (attr & CA_NBEG && x <= Lxlm)
	erret2 (E_MOVBEG);
#endif
      if (attr & CA_CHANGE)
	{
	  if (attr & (CA_NEND | CA_NBEG))
	    {
	      if (attr & CA_NBEG)
		x--;
	      if (x >= Lxre)
		erret2 (E_EDTEND);
	      if (x < Lxle)
		erret2 (E_EDTBEG);
	    }
	}
      if (qKbBreak ())
	erret2 (E_KBREAK);
      (*cp->cfunc) ();
      if (attr & CA_CHANGE)
	{
	  Lchange = TRUE;
	  if (Ltxt == Ttxt)
	    stModification (STM_YES);
	}
    }
  ccrov = 0;
  if (typ & RQ_WAIT)
    goto loop;
  erret (E_OK);
ret2:
  if (!(attr & CA_RPT))
    KbCount = r;
ret:
#ifdef NO_MARGIN_BELL
  if (typ & RQ_WAIT && !Event[x])
    {
      if (!IsMargin)
	scbell ();
      else
	IsMargin = 0;
      goto loop;
    }
#else
  if (typ & RQ_WAIT && !Event[x])
    {
      scbell ();
      goto loop;
    }
#endif
  KbWaitIn = kboldhook;
  excptr = nextexc;
  return (x);
}

static void
lekbwait ()
{
  register wnd *w;
  register txt *t;

  wadjust (w = Lwnd, Lx, Ly);
  if (BlockMark)
    cpblockmark ();
  wpos (w, Lx, Ly);
/* stCXYSym( Lx+1, Ty+1 ); */
  stCXYSym (Lx, Ly);
}

static bool
lcword ()
{
  register small x;
  if (!word_separators[(unsigned char) Lebuf[x = Lx]])
    {
      while (x > Lxlm)
	if (word_separators[(unsigned char) Lebuf[--x]])
	  {
	    x++;
	    break;
	  }
      lxl = x;
      while (x < Lxrm)
	if (word_separators[(unsigned char) Lebuf[++x]])
	  break;
      Lx = x;
      return (TRUE);
    }
  return (FALSE);
}

static void
lcdword ()
{
  register small xl, xr;
  xl = min (Lxre, Lxrm);
  for (xr = Lx - 1; xr < xl;)
    if ((char) Lebuf[++xr] != ' ') /* delete trailing spaces */
      break;

  if (Lxle > (xl = lxl))
    xl = Lxle;
  if (Lxre < xr)
    xr = Lxre;
  if (xl >= xr)
    exc (E_EDTBEG);
  llmove (Lx = xl, Lxre, xl - xr, NIL);
}

static void
lechr2 ()
{
  register char c;
  c = (char) lchar;
  if (c == '\002' || c == '\015' || c == '\012' || c == '\032')
    exc (E_ILCHAR);
  if (Lrins)
    leic2 ();
  else
    llchar ();
  Lx++;
}

static void
leic2 ()
{
  if ((char) Lebuf[Lxre - 1] != ' ')
    exc (E_EDTEND);
  llmove (Lx, Lxre, 1, &lchar);
}
void
leic ()
{
  lchar = ' ';
  leic2 ();
}

void
lebs ()
{
  Lx--;
  if (Lrins)
    ledc ();
  else
    {
      lchar = ' ';
      llchar ();
    }
}

void
lechar ()
{
  lchar = ctotc ((char) KbCode) | Lattr;
  lechr2 ();
}

void
lespchar ()
{
  if (KbRadix == 0)
    exc (E_ILCHAR);
  lchar = KbCount;
  lechr2 ();
}

void
leescchr ()
{
  lchar = ESC;
  lechr2 ();
}
void
lehfchar ()
{
  lchar = HYPHEN;
  lechr2 ();
}
void
ledc ()
{
  llmove (Lx, Lxre, -1, NIL);
}
void
ledeol ()
{
  llmove (Lx, Lxre, REPLACE, NIL);
}
void
ledbgol ()
{
  llmove (Lxle, Lxre, Lxle - Lx, NIL);
  Lx = Lxle;
}

void
ledword ()
{
  if (!lcword ())
    {
      /*    :      */
      char ch = Lebuf[Lx];
      int xl, xr;

      lxl = Lx;
      xl = min (Lxre, Lxrm);
      for (xr = Lx - 1; xr < xl;)
	if ((char) Lebuf[++xr] != ch)
	  break;

      if (Lxle > (xl = lxl))
	xl = Lxle;
      if (Lxre < xr)
	xr = Lxre;
      if (xl >= xr)
	exc (E_EDTBEG);

      llmove (Lx = xl, Lxre, xl - xr, NIL);
    }
  else
    lcdword ();
}

void
ledlword ()
{
  register small x, xl;
  xl = max (Lxle, Lxlm);
  for (x = Lx; x > xl;)
    if (word_separators[(unsigned char) Lebuf[--x]])
      {
	x++;
	if (x == Lx)
	  {
	    /*    :      */
	    char ch = Lebuf[Lx - 1];

	    while (x > xl && ch == Lebuf[--x]);
	    if (x > xl) /*    */
	      x++;
	  }
	break;
      }
  if (Lrins)
    llmove (x, Lxre, x - Lx, NIL);
  else
    llmove (x, Lx, REPLACE, NIL);
  Lx = x;
}

void
lecentr ()
{
  register small xl, xr, xll, xrr;
  xl = max (Lxle, Lxlm);
  xr = min (Lxre, Lxrm);
  xll = xl = max (fplm, xl);
  xrr = xr = min (fprm, xr);
  do
    {
      if (xl >= xrr)
	return;
    }
  while ((char) Lebuf[xl++] == ' ');
  xl = xl - xll - 1;
  do
    {
      if (xr <= xll)
	return;
    }
  while ((char) Lebuf[--xr] == ' ');
  xr = xrr - xr - 1;
  if (xr = (xr - xl) >> 1)
    llmove (xll, xrr, xr, NIL);
}


#ifdef KOI8
static void
leclanguage (unsigned char *pat)
{
  register tchar tc;
  register unsigned char c;

  c = (tc = Lebuf[Lx]);
  if ((c >= pat[0] && c <= pat[1]) || (c >= pat[2] && c <= pat[3]))
    c = keytable[c];

  lchar = (tc & ~0377) | (c & 0377);
  llchar ();
  Lx++;
}

void
leccup ()
{
  leccase ("az\340\040");
}
void
leccdwn ()
{
  leccase ("AZ\040\340");
}
void
leccrus ()
{
  leclanguage ("azAZ");
}
void
lecclat ()
{
  leclanguage ("");
}
#else /* ! KOI8 */
void
leccup ()
{
  leccase ("az\340 \040");
}
void
leccdwn ()
{
  leccase ("AZ\040\377\340");
}
void
leccrus ()
{
  leccase ("az\140AZ\240");
}
void
lecclat ()
{
  leccase ("\240\140");
}
#endif /* KOI8 */

static void
leccase (char *pat)
{
  register tchar tc;
  register char c, *p;
  c = (tc = Lebuf[Lx]);
  if (p = pat, c >= *p++ && c <= *p++)
    c += *p++;
  else if (p = pat + 3, c >= *p++ && c <= *p++)
    c += *p++;
  lchar = (tc & ~0377) | (c & 0377);
  llchar ();
  Lx++;
}

void
lecattr ()
{
#ifndef NO_ATTRIBUTES
  lchar = (Lebuf[Lx] & 0377) | Lattr;
  llchar ();
  Lx++;
#endif /* NO_ATTRIBUTES */
}


void
lanone ()
{
#ifndef NO_ATTRIBUTES
  Lattr = 0;
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
lalight ()
{
#ifndef NO_ATTRIBUTES
  Lattr ^= (AC_LIGHT << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
laitalic ()
{
#ifndef NO_ATTRIBUTES
  Lattr ^= (AC_ITALIC << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
launderl ()
{
#ifndef NO_ATTRIBUTES
  Lattr ^= (AC_UNDERL << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
labold ()
{
#ifndef NO_ATTRIBUTES
  Lattr ^= (AC_BOLD << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
lasuper ()
{
#ifndef NO_ATTRIBUTES
  Lattr &= ~(AC_SUBSTR << 8);
  Lattr ^= (AC_SUPER << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
lasubstr ()
{
#ifndef NO_ATTRIBUTES
  Lattr &= ~(AC_SUPER << 8);
  Lattr ^= (AC_SUBSTR << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}
void
lainvert ()
{
#ifndef NO_ATTRIBUTES
  Lattr ^= (AC_INVERT << 8);
  stCAttr (Lattr);
#endif /* NO_ATTRIBUTES */
}

void
leright ()
{
  Lx++;
}
void
leleft ()
{
  Lx--;
}
void
lebeg ()
{
  Lx = Lxlm;
}

void
leend ()
{
  register small x;
  for (x = Lxrm; x > Lxlm;)
    if ((char) Lebuf[--x] != ' ')
      {
	x++;
	break;
      }
  Lx = x;
}

void
letab ()
{
  register small x;
  for (x = Lx; x < Lxrm;)
    if (tabuf[++x] & TABSTOP)
      break;
  Lx = x;
}

void
leltab ()
{
  register small x;
  for (x = Lx; x > Lxlm;)
    if (tabuf[--x] & TABSTOP)
      break;
  Lx = x;
}

void
lenword ()
{
  register small x;
  for (x = Lx; ++x < Lxrm;)
    if ( ! word_separators[(unsigned char) Lebuf[x]]
	&& word_separators[(unsigned char) Lebuf[x - 1]])
      goto ok;
  while (word_separators[(unsigned char) Lebuf[--x]])
    if (x == Lx)
      goto ng;
  if (++x < Lxrm)
    x++;
ok:Lx = x;
ng:{
  }
}

void
lepword ()
{
  register small x;
  for (x = Lx; --x > Lxlm;)
    if ( ! word_separators[(unsigned char) Lebuf[x]]
	&& word_separators[(unsigned char) Lebuf[x - 1]])
      break;
  Lx = x;
}


static void
qccem ()
{
  if (ccempt)
    {
      cclen = 0;
      ccempt = FALSE;
    }
}

void
lecchar ()
{
  qccem ();
  if (cclen == MAXLPAC)
    exc (E_CCFUL);
  ccbuf[cclen++] = Lebuf[Lx++];
#ifdef X11
  xiMarkSelection ();
#endif
}

void
lecdchar ()
{
  lecchar ();
  Lx--;
  ledc ();
}

local void
lecword1 ()
{
  register small x;
  qccem ();
  if (lcword ())
    {
#if 0
      /*   -    */
      if (cclen != 0 && cclen != MAXLPAC)
	ccbuf[cclen++] = ' ';
#endif
      for (x = lxl; x < Lx;)
	{
	  if (cclen == MAXLPAC)
	    goto fail;
	  ccbuf[cclen++] = Lebuf[x++];
	}
#ifdef X11
      xiMarkSelection ();
#endif
    }
  else
  fail:exc (E_CCFUL);
}

void
lecword ()
{
  /* ,   ' ',      ,
       . */
  if (word_separators[(unsigned char) Lebuf[Lx]])
    lecchar ();
  else
    lecword1 ();
#if 0
  /*      , ..     */
  lenword ();
#endif
}

void
lecdword ()
{
  if (word_separators[(unsigned char) Lebuf[Lx]])
    lecdchar ();
  else
    {
      lecword1 ();
      lcdword ();
    }
}

void
lepchar ()
{
  ccempt = TRUE;
  while (ccrov < cclen)
    {
      if (Lx >= Lxre)
	exc (E_EDTEND);
      if (Lx < Lxle)
	exc (E_EDTBEG);
      lchar = ccbuf[ccrov++];
      lechr2 ();
    }
  ccrov = 0;
}

void
leccchr ()
{
  cclen = 0;
  ccempt = FALSE;
}

void
lerins ()
{
  Lrins = TRUE;
  stCMode (STM_INSERT);
}
void
lerrep ()
{
  Lrins = FALSE;
  stCMode (STM_REPLACE);
}

local void
llchar ()
{
  register small x;
  register wnd *w;
  x = Lx;
  lundoadd (Ltxt, x, x + 1, REPLACE, (char *) (Lebuf + x), (char *) &lchar);
  if ((Lebuf[x++] = lchar) != ' ')
    {
      if (Lleng < x)
	Lleng = x;
    }
  else if (Lleng == x)
    Lleng = leleng ();
  w = Lwnd;
  wxmin = Lx - w->wtx;
  wxmax = wxmin + 1;
  wymin = Ly - w->wty;
  wymax = wymin + 1;
  wrdrw (w);
#ifdef X11
/*wpos( w, Lx - w->wtx + 1, Ly - w->wty );*/
  wpos (w, Lx + 1, Ly);
#endif
}

static tchar *
lefill (register tchar * d, register small l, register tchar * s)
{
  for (; l; l--)
    *d++ = (s != NIL) ? *s++ : (tchar) ' ';
  return (d);
}

static small
leleng ()
{
  return (lstrlen (MAXLPAC, Lebuf));
}

small
lstrlen (small lmax, register tchar * p0)
{
  register tchar *p;
  if (p0 == NIL)
    return (0);
  for (p = p0 + lmax; p > p0;)
    if (*--p != (tchar) ' ')
      {
	p++;
	break;
      }
  return (p - p0);
}

void
llmove (register small xl, register small xr, register small dx, tchar * ns)
{
  register wnd *w;
  if (dx == REPLACE)
    {
      lundoadd (Ltxt, xl, xr, dx, (char *) (Lebuf + xl), (char *) ns);
      lefill (Lebuf + xl, xr - xl, ns);
    }
  else
    {
      if (dx == 0)
	return;
      if (dx > 0)
	{
	  lundoadd (Ltxt, xl, xr, dx, (char *) (Lebuf + xr - dx),
		    (char *) ns);
	  blkmov ((char *) (Lebuf + xl), (char *) (Lebuf + xl + dx),
		  (xr - xl - dx) * sizeof (tchar));
	  lefill (Lebuf + xl, dx, ns);
	}
      else
	{
	  lundoadd (Ltxt, xl, xr, dx, (char *) (Lebuf + xl), (char *) ns);
	  blkmov ((char *) (Lebuf + xl - dx), (char *) (Lebuf + xl),
		  (xr - xl + dx) * sizeof (tchar));
	  lefill (Lebuf + xr + dx, -dx, ns);
	}
    }
  Lleng = leleng ();
  w = Lwnd;
  wxmin = xl - w->wtx;
  wxmax = xr - w->wtx;
  wdx = dx;
  wymin = Ly - w->wty;
  wymax = wymin + 1;
  wdy = 0;
  wroll (w);
}

tchar *
LeInfo (register wnd * w, small y, register small * pl)
{
  if ((w->wty + y) == Ly && (*pl = (Lleng - w->wtx)) > 0)
    return (Lebuf + w->wtx);
  *pl = 0;
  return (NIL);
}

void
Leload (small l)
{
  blktspac (Lebuf + l, MAXLPAC - l);
  Lchange = FALSE;
}

void
blktspac (register tchar * p, register small l)
{
  if (l)
    do
      {
	*p++ = (tchar) ' ';
      }
    while (--l);
}

/* Copy save buffer into BUF of length LEN */
void
LeGetSaveBuffer (char * buf, int len)
{
  int i;
  int n = (cclen >= len ? len - 1 : cclen);

  for (i = 0; i < n; i++)
    buf[i] = ccbuf[i];
  buf[i] = '\0';
  ccempt = TRUE;
}
