#include "cmn.h"
#include "kblinux.h"

#define MAXTABS 50
#define MAXLENTAB 200
#define FILE_NAME "ncsa.cm"

short origin_table[] = {
                        /* LINE EDIT COMMANDS */
LE_RIGHT   ,RIGHT,             -1,/*  0401 */   /* move to right */
LE_RIGHT   ,N6,                -1,/*  0401 */   /* move to right */
LE_LEFT    ,LEFT,              -1,/*  0402 */   /* move to left */
LE_LEFT    ,N4,                -1,/*  0402 */   /* move to left */
LE_DEOL    ,CTRL_D,            -1,/*  0403 */   /* delete end of line */
LE_DBGOL   ,CTRL_U,            -1,/*  0404 */   /* delete beginning of line */
LE_BEG     ,HOME,              -1,/*  0405 */   /* move at the beginning of line */
LE_BEG     ,CTRL_LEFT,         -1,/*  0405 */   /* move at the beginning of line */
LE_BEG     ,N7,                -1,/*  0405 */   /* move at the beginning of line */
LE_BEG     ,N5,LEFT,           -1,/*  0405 */   /* move at the beginning of line */
LE_BEG     ,N5,N4,             -1,/*  0405 */   /* move at the beginning of line */
LE_END     ,END,               -1,/*  0406 */   /* move at the end of line */
LE_END     ,CTRL_RIGHT,        -1,/*  0406 */   /* move at the end of line */
LE_END     ,N1,                -1,/*  0406 */   /* move at the end of line */
LE_END     ,N5,RIGHT,          -1,/*  0406 */   /* move at the end of line */
LE_END     ,N5,N6,             -1,/*  0406 */   /* move at the end of line */
LE_IC      ,INSERT,            -1,/*  0407 */   /* insert a space */
LE_IC      ,N0,                -1,/*  0407 */   /* insert a space */
LE_DC      ,DELETE,            -1,/*  0410 */   /* delete character */
LE_DC      ,DOT,               -1,/*  0410 */   /* delete character */
LE_BS      ,BACKSPACE,         -1,/*  0411 */   /* backspace (delete left character) */
LE_BS      ,CTRL_H,            -1,/*  0411 */   /* backspace (delete left character) */
LE_TAB     ,TAB,               -1,/*  0412 */   /* right tabulation */
LE_LTAB    ,SHIFT_TAB,         -1,/*  0413 */   /* left tabulation */
LE_NWORD   ,SHIFT_RIGHT,       -1,/*  0414 */   /* move on a word to right */
LE_PWORD   ,SHIFT_LEFT,        -1,/*  0415 */   /* move on a word to left */
LE_CCHAR   ,F1,                -1,/*  0416 */   /* save character */
LE_CDCHAR  ,SHIFT_F1,          -1,/*  0417 */   /* save and delete character */
LE_PCHAR   ,F2,                -1,/*  0420 */   /* restore character */
LE_CCCHR   ,SHIFT_F2,          -1,/*  0421 */   /* clear character save buffer */
LE_CWORD   ,CTRL_F1,           -1,/*  0422 */   /* save word */
LE_CDWORD  ,CTRL_SHIFT_F1,     -1,/*  0423 */   /* save and delete word */
LE_DWORD   ,CTRL_A,            -1,/*  0424 */   /* delete current word with trailing spaces */
LE_DLWORD  ,SHIFT_BACKSPACE,   -1,/*  0425 */   /* delete previous word */
LE_CENTR   ,CTRL_T,            -1,/*  0426 */   /* center current line */
LE_RINS    ,CTRL_INSERT,       -1,/*  0427 */   /* set insert mode */
LE_RREP    ,CTRL_DELETE,       -1,/*  0430 */   /* set replace mode */
LE_CCUP    ,SHIFT_UP,          -1,/*  0431 */   /* current letter -> upper */
LE_CCDWN   ,SHIFT_DOWN,        -1,/*  0432 */   /* current letter -> lower */
LE_CCRUS   ,ALT_UP,            -1,/*  0433 */   /* current letter -> russian (national) */
LE_CCLAT   ,ALT_DOWN,          -1,/*  0434 */   /* current letter -> english */
LE_CATTR   ,CTRL_SHIFT_SPACE,  -1,/*  0435 */   /* current letter -> current attribute */
LE_SPCHAR  ,ESC,CTRL_X,        -1,/*  0436 */   /* enter special character */
LE_ESCCHR  ,ALT_ESC,           -1,/*  0437 */   /* enter escape character (033) */
LE_HFCHAR  ,ALT_MINUS,         -1,/*  0440 */   /* enter hyphen */
                        /* ATTRIBUTES */
LA_NONE    ,CTRL_SPACE,        -1,/*  0441 */   /* input attributes = clear */
LA_LIGHT   ,CTRL_L,            -1,/*  0442 */   /* input attributes |= light */
LA_ITALIC  ,CTRL_K,            -1,/*  0443 */   /* input attributes |= italic */
LA_UNDERL  ,CTRL_P,            -1,/*  0444 */   /* input attributes |= underline */
LA_BOLD    ,CTRL_V,            -1,/*  0445 */   /* input attributes |= bold */
LA_SUPER   ,CTRL_W,            -1,/*  0446 */   /* input attributes |= superscript */
LA_SUBSCR  ,CTRL_N,            -1,/*  0447 */   /* input attributes |= subscript */
LA_INVERT  ,CTRL_J,            -1,/*  0450 */   /* input attributes |= inverse */
                        /* UNDO */
TE_UNDO    ,CTRL_BACKSPACE,    -1,/*  0451 */   /* undo */
TE_UNUNDO  ,CTRL_RETURN,       -1,/*  0452 */   /* undo undo */
TE_SUNDO   ,ALT_CTRL_BACKSPACE,-1,/*  0453 */   /* slow undo */
TE_SUNUNDO ,ALT_CTRL_RETURN,   -1,/*  0454 */   /* slow undo undo */
                        /* FILE MODE */
TE_SRO     ,N5, CTRL_R,        -1,/*  0455 */   /* set read-only mode */
TE_SRW     ,N5, CTRL_E,        -1,/*  0456 */   /* cancel read-only mode */

TE_SWIGHT  ,N5, CTRL_W,        -1,/*  0457 */   /* set width of text */
                        /* MARKER */
TE_SMARK   ,CTRL_TAB,          -1,/*  0460 */   /* set marker */
TE_CMARK   ,CTRL_SHIFT_TAB,    -1,/*  0461 */   /* swap marker and cursor */
                        /* TABULATION / FORMATTING PATTRENS */
TE_PTAB    ,N5, TAB,           -1,/*  0462 */   /* edit tabulation pattern */
TE_PFORM   ,N5, F9,            -1,/*  0463 */   /* edit formatting pattern */
                        /* TEXT EDIT COMMANDS */
TE_UP      ,UP,                -1,/*  0464 */   /* up */
TE_UP      ,N8,                -1,/*  0464 */   /* up */
TE_DOWN    ,DOWN,              -1,/*  0465 */   /* down */
TE_DOWN    ,N2,                -1,/*  0465 */   /* down */
TE_CUP     ,N5, UP,            -1,/*  0466 */   /* initiate upward motion */
TE_CUP     ,CTRL_UP,           -1,/*  0466 */   /* initiate upward motion */
TE_CUP     ,N5, N8,            -1,/*  0466 */   /* initiate upward motion */
TE_CDOWN   ,N5, DOWN,          -1,/*  0467 */   /* initiate downward motion */
TE_CDOWN   ,CTRL_DOWN,         -1,/*  0467 */   /* initiate downward motion */
TE_CDOWN   ,N5, N2,            -1,/*  0467 */   /* initiate downward motion */
TE_TBEG    ,CTRL_SHIFT_UP,     -1,/*  0470 */   /* go to top of file */
TE_TEND    ,CTRL_SHIFT_DOWN,   -1,/*  0471 */   /* go to end of file */

TE_IL      ,SHIFT_INSERT,      -1,/*  0472 */   /* insert blank line */
TE_DL      ,SHIFT_DELETE,      -1,/*  0473 */   /* delete current line */
TE_CR      ,RETURN ,           -1,/*  0474 */   /* move at the beginning of next line */
TE_RCR     ,ALTGR_RETURN,      -1,/*  0475 */   /* move at the beginning of previous line */
TE_SENTR   ,N5,STAR,           -1,/*  0476 */   /* input search and replace pattern */
TE_SUP     ,SHIFT_STAR,        -1,/*  0477 */   /* search up */
TE_SDOWN   ,STAR,              -1,/*  0500 */   /* search down */
TE_CENTR   ,CTRL_E,            -1,/*  0501 */   /* move current line to middle of screen */
TE_CLIN    ,F3,                -1,/*  0502 */   /* save current line */
TE_CDLIN   ,SHIFT_F3,          -1,/*  0503 */   /* save current line and delete it */
TE_PLIN    ,F4,                -1,/*  0504 */   /* restore line(s) */
TE_CCLIN   ,SHIFT_F4,          -1,/*  0505 */   /* clear line save buffer */
TE_BLIN    ,N5,RETURN,         -1,/*  0506 */   /* cut line with horizontal displacement */
TE_SLIN    ,N5,INSERT,         -1,/*  0507 */   /* cut line without horizontal displacement */
TE_NBLIN   ,N5,BACKSPACE,      -1,/*  0510 */   /* paste line with horizontal displacement */
TE_NSLIN   ,N5,DELETE,         -1,/*  0511 */   /* paste line without horizontal displacement */
                        /* SCREENS */
TE_IPAGE   ,N5,CTRL_H,         -1,/*  0512 */   /* set page height */
TE_NPAGE   ,PageDown,          -1,/*  0513 */   /* next screen */
TE_NPAGE   ,N3,                -1,/*  0513 */   /* next screen */
TE_PPAGE   ,PageUp,            -1,/*  0514 */   /* previous screen */
TE_PPAGE   ,N9,                -1,/*  0514 */   /* previous screen */
                        /* BLOCKS */
TE_MCBLOCK ,F5,                -1,/*  0515 */   /* mark begin of block/save block */
TE_CDBLOCK ,CTRL_F5,           -1,/*  0516 */   /* save and delete block */
TE_CCBLOCK ,SHIFT_F5,          -1,/*  0517 */   /* save and clear block */
TE_PBLOCK  ,F6,                -1,/*  0520 */   /* restore block over text */
TE_PIBLOCK ,CTRL_F6,           -1,/*  0522 */   /* insert saved block */
TE_PRBLOCK ,SHIFT_F6,          -1,/*  0521 */   /* restore block with reverse */
TE_PIRBLOCK,CTRL_SHIFT_F6,     -1,/*  0523 */   /* insert block with reverse */
TE_BLKEXEC ,N5,ENTER,          -1,/*  1021 */   /* execute OS commands from block */
TE_BLKMOD  ,CTRL_SHIFT_F5,     -1,/*  ???? */   /* modify block */
                        /* DRAWING LINES, SEARCH */
TE_LPRINT  ,                   -1,/*  0524 */   /* ???? */
TE_FPRINT  ,                   -1,/*  0525 */   /* ???? */
TE_FORMAT  ,F9,                -1,/*  0526 */   /* reformat within intrinsic bounds */
TE_PFORMAT ,SHIFT_F9,          -1,/*  0527 */   /* reformat by pattern */
TE_LDUP    ,ALTGR_UP,          -1,/*  0530 */   /* draw line up */
TE_LDDOWN  ,ALTGR_DOWN,        -1,/*  0531 */   /* draw line down */
TE_LDLEFT  ,ALTGR_LEFT,        -1,/*  0532 */   /* draw line to left */
TE_LDRIGHT ,ALTGR_RIGHT,       -1,/*  0533 */   /* draw line to right */
TE_RUP     ,ALT_CTRL_STAR,    -1,/*  0534 */   /* search and replace up */
TE_RDOWN   ,ALT_STAR,          -1,/*  0535 */   /* search and replace down */
                        /* ???? */
TE_PPRINT  ,                   -1,/*  0536 c */ /* ???? */
TE_IPRINT  ,                   -1,/*  0537 ( */ /* ???? */
TE_RSPELL  ,                   -1,/*  0540 / */ /* ???? */
TE_LSPELL  ,                   -1,/*  0541   */ /* ???? */
TE_SPADD   ,                   -1,/*  0542 $ */ /* ???? */
TE_SPREM   ,                   -1,/*  0543   */ /* ???? */
                        /* FILE NAVIGATION */
TE_HELP    ,CTRL_QUESTION,     -1,/*  0544  help */     /* help */
TE_MSGS    ,                   -1,/* 01013 */   /* make message to developers */
TE_FENTR   ,ESC ,DOWN,         -1,/*  0545 */   /* enter to file */
TE_FENTR   ,ESC ,N2,           -1,/*  0545 */   /* enter to file */
TE_F2ENTR  ,ESC ,ESC,          -1,/*  0546 */   /* obtain filename and enter file */
TE_F2ENTR  ,ESC ,CTRL_F,       -1,/*  0546 */   /* obtain filename and enter file */
TE_FEXEC   ,ESC,ENTER,         -1,/*  0547 */   /* execute current file */
TE_EXEC    ,ESC,CTRL_ENTER,    -1,/*  0547 */   /* obtain filename and execute file */
TE_SHELL   ,ENTER,             -1,/*  0550 */   /* enter to OS shell */
TE_INFILE  ,ESC,RIGHT,         -1,/*  0551 */   /* move to filename field in micros.dir */
TE_INFILE  ,ESC,N6,            -1,/*  0551 */   /* move to filename field in micros.dir */
TE_OUTFILE ,ESC,LEFT,          -1,/*  0552 */   /* move out filename field in micros.dir */
TE_OUTFILE ,ESC,N4,            -1,/*  0552 */   /* move out filename field in micros.dir */

TE_MODFILE ,ESC,CTRL_U,        -1,              /* get and edit file attribute of filename in cursor position */

TE_UPDATE  ,N5,ESC,            -1,/*  0553 */   /* save file without exit */
TE_EXIT    ,ESC,UP,            -1,/*  0554 */   /* save and exit file */
TE_EXIT    ,ESC,N8,            -1,/*  0554 */   /* save and exit file */
TE_EXOLD   ,ESC,CTRL_C,        -1,/*  0555 */   /* exit without save */
TE_EXOLD   ,ESC,CTRL_Q,        -1,/*  0555 */   /* exit without save */
TE_EXBACK  ,ESC,BACKSPACE,     -1,/*  0556 */   /* exit file with back-up copy */
TE_QUIT    ,ESC,CTRL_T,        -1,/*  0557 */   /* quit from editor */
TE_CRASH   ,ESC,CTRL_Y,        -1,/*  0557 */   /* crash editor */
TE_NERROR  ,F11,               -1,/*  0560 */   /* move to next error */
TE_PERROR  ,SHIFT_F11,         -1,/*  0561 */   /* move to previous error */
TE_ERROFF  ,CTRL_F11,          -1,/* 01022 */   /* exit error mode */
TE_STATUS  ,CTRL_S,            -1,/* 01023 */   /* toggle status line */
                        /* SEVERAL TEXTS */
TE_ZOOM    ,CTRL_R,            -1,/* 01024 */   /* zoom window */
TE_UFORK   ,NumLock,UP,        -1,/*  0562 */   /* vertical split window and move to up window */
TE_DFORK   ,NumLock,DOWN,      -1,/* 01003 */   /* vertical split window and move to down window */
TE_RFORK   ,NumLock,RIGHT,     -1,/* 01004 */   /* horisontal split window and move to right window */
TE_LFORK   ,NumLock,LEFT,      -1,/*  0563 */   /* horisontal split window and move to left window */
TE_UWINDOW ,ALT_CTRL_UP,       -1,/*  0564 */   /* move to up window */
TE_DWINDOW ,ALT_CTRL_DOWN,     -1,/*  0565 */   /* move to down window */
TE_RWINDOW ,ALT_CTRL_RIGHT,    -1,/*  0566 */   /* move to right window */
TE_LWINDOW ,ALT_CTRL_LEFT,     -1,/*  0567 */   /* move to left window */
                        /* DISPLAY */
TK_BREAK   ,CTRL_C,            -1,/*  0570 */   /* interrupt a long command */
TK_BREAK   ,CTRL_Q,            -1,/*  0570 */   /* interrupt a long command */
TK_CTRLO   ,CTRL_Z,            -1,/*  0571 */   /* block/deblock drawing */
TK_REPEAT  ,N5,N5,             -1,/*  0573 */   /* repeat previous command */
TK_BRUS    ,SHIFT_PLUS,        -1,/*  0574 */   /* set big russian */
TK_SRUS    ,PLUS,              -1,/*  0575 */   /* set small russian */
TK_BLAT    ,SHIFT_MINUS,       -1,/*  0576 */   /* set big english */
TK_SLAT    ,MINUS,             -1,/*  0577 */   /* set small english */
                        /* MACRO */
TK_SM+0    ,SHIFT_F7,          -1,/*  0600 */   /* initiate capturing of macro */
TK_EM+0    ,F7,                -1,/*  0700 */   /* finish capturing of macro / excute macro */
TK_SM+1    ,CTRL_SHIFT_1,      -1,/*  0600 */   /* initiate capturing of macro 1 */
TK_EM+1    ,CTRL_1,            -1,/*  0700 */   /* finish capturing of macro 1 / excute macro 1 */
TK_SM+2    ,CTRL_SHIFT_2,      -1,/*  0600 */   /* initiate capturing of macro 2 */
TK_EM+2    ,CTRL_2,            -1,/*  0700 */   /* finish capturing of macro 2 / excute macro 2 */
TK_SM+3    ,CTRL_SHIFT_3,      -1,/*  0600 */   /* initiate capturing of macro 3 */
TK_EM+3    ,CTRL_3,            -1,/*  0700 */   /* finish capturing of macro 3 / excute macro 3 */
TK_SM+4    ,CTRL_SHIFT_4,      -1,/*  0600 */   /* initiate capturing of macro 4 */
TK_EM+4    ,CTRL_4,            -1,/*  0700 */   /* finish capturing of macro 4 / excute macro 4 */
TK_SM+5    ,CTRL_SHIFT_5,      -1,/*  0600 */   /* initiate capturing of macro 5 */
TK_EM+5    ,CTRL_5,            -1,/*  0700 */   /* finish capturing of macro 5 / excute macro 5 */
TK_SM+6    ,CTRL_SHIFT_6,      -1,/*  0600 */   /* initiate capturing of macro 6 */
TK_EM+6    ,CTRL_6,            -1,/*  0700 */   /* finish capturing of macro 6 / excute macro 6 */
TK_SM+7    ,CTRL_SHIFT_7,      -1,/*  0600 */   /* initiate capturing of macro 7 */
TK_EM+7    ,CTRL_7,            -1,/*  0700 */   /* finish capturing of macro 7 / excute macro 7 */
TK_SM+8    ,CTRL_SHIFT_8,      -1,/*  0600 */   /* initiate capturing of macro 8 */
TK_EM+8    ,CTRL_8,            -1,/*  0700 */   /* finish capturing of macro 8 / excute macro 8 */
TK_SM+9    ,CTRL_SHIFT_9,      -1,/*  0600 */   /* initiate capturing of macro 9 */
TK_EM+9    ,CTRL_9,            -1,/*  0700 */   /* finish capturing of macro 9 / excute macro 9 */
TK_SM+10   ,CTRL_SHIFT_0,      -1,/*  0600 */   /* initiate capturing of macro 0 */
TK_EM+10   ,CTRL_0,            -1,/*  0700 */   /* finish capturing of macro 0 / excute macro 0 */
TK_SM+11   ,SHIFT_F8,          -1,/*  0600 */   /* initiate capturing of macro 11 */
TK_EM+11   ,F8,                -1,/*  0700 */   /* finish capturing of macro 11 / excute macro 11 */
                        /* DEBUG ???? */
TD_SDEQ    ,                   -1,/*  01001*/   /* show deqs */
TD_STXT    ,                   -1,/*  01002*/   /* show texts */
                        /* COUNTERS, DATE, TIME */
TK_CSET    ,ESC,CTRL_PageDown, -1,              /* set counter value */
TK_CADD    ,CTRL_PageUp,       -1,              /* increment counter value */
TK_CSUB    ,CTRL_PageDown,     -1,              /* decrement counter value */
TK_CPUT    ,ESC,CTRL_PageUp,   -1,              /* put counter value */
TK_DATE    ,N5,CTRL_D,         -1,/*  01005*/   /* put current date */
TK_TIME    ,N5,CTRL_T,         -1,/*  01006*/   /* put current time */
                        /* MACRO AND FILES */
MC_SAVE    ,ESC,SLASH,SLASH,DOWN,-1,/*  01007*/   /* save macros in file */
MC_LOAD    ,ESC,SLASH,SLASH,UP,-1,/*  01010*/   /* load macros from file */
MC_2SAVE   ,ESC,SLASH, DOWN,   -1,/*  01011*/   /* obtain filename and save macros */
MC_2LOAD   ,ESC,SLASH, UP,     -1,/*  01012*/   /* obtain filename and load macros */
                        /* F-PRACTICUM COMMANDS */
TE_OHELP   ,                   -1,/*  0544  help */     /* F-practicum help */
TE_OHELP   ,                   -1,/*  0544  help */     /* F-practicum help */
EX_EXEC    ,                   -1,/*  02030*/   /* F-practicum execute */
EX_CEXEC   ,                   -1,/*  02031*/   /* F-practicum continuous execute */
EX_FEXEC   ,                   -1,/*  02032*/   /* F-practicum file execute */

MC_INCL    ,                   -1,/*  02000*/   /* F-practicum input */
MC_INCL    ,                   -1,/*  02000*/   /* F-practicum input */
MC_PROC    ,                   -1,/*  02000*/   /* F-practicum input procedure */
MC_PROC    ,                   -1,/*  02000*/   /* F-practicum input procedure */
MC_SUBR    ,                   -1,/*  02000*/   /* F-practicum input subroutine */
MC_SUBR    ,                   -1,/*  02000*/   /* F-practicum input subroutine */
MC_FUNC    ,                   -1,/*  02000*/   /* F-practicum input function */
MC_FUNC    ,                   -1,/*  02000*/   /* F-practicum input function */
MC_WHIL    ,                   -1,/*  02000*/   /* F-practicum input while statement */
MC_WHIL    ,                   -1,/*  02000*/   /* F-practicum input while statement */
MC_DO      ,                   -1,/*  02000*/   /* F-practicum input do statement */
MC_DO      ,                   -1,/*  02000*/   /* F-practicum input do statement */
MC_IF      ,                   -1,/*  02000*/   /* F-practicum input if statement */
MC_IF      ,                   -1,/*  02000*/   /* F-practicum input if statement */

DB_DUMP    ,                   -1,/*  02000*/   /* F-practicum show dump */
DB_DUMP    ,                   -1,/*  02000*/   /* F-practicum show dump */
DB_TRACE   ,                   -1,/*  02000*/   /* F-practicum show trace */
DB_TRACE   ,                   -1,/*  02000*/   /* F-practicum show trace */

-1};

short special_table[] = {

TK_ESC     ,ESC,               /*  0572  Esc */
-1};

#include "gentab.c"
