#include "cmn.h"
#include "kbpcanew-ANSI.h"

#define MAXTABS 64
#define MAXLENTAB 240
#define FILE_NAME "pcanew-ANSI.cm"

short origin_table[] = {

LE_RIGHT   ,RIGHT,             -1,/*  0401  move right */
LE_LEFT    ,LEFT,              -1,/*  0402  move left */
LE_DEOL    ,CTRL_D,            -1,/*  0403  erase end of line */
LE_DBGOL   ,CTRL_U,            -1,/*  0404  collapse beginning of line */
LE_BEG     ,Home, LEFT,        -1,/*  0405  move to end of line */
LE_END     ,Home, RIGHT,       -1,/*  0406  move to beginning of line */
LE_IC      ,Insert,            -1,/*  0407  insert spacebar */
LE_DC      ,Delete,            -1,/*  0410  delete current character */
LE_BS      ,BS,                -1,/*  0411  delete previous character */
LE_TAB     ,Tab,               -1,/*  0412  move to next tab position */
LE_LTAB    ,CTRL_BSL,          -1,/*  0413  move to previous tab position */
LE_NWORD   ,CTRL_A, RIGHT,     -1,/*  0414  move to next word */
LE_PWORD   ,CTRL_A, LEFT,      -1,/*  0415  move to previous word */
LE_CCHAR   ,F1,                -1,/*  0416  add character to buffer */
LE_CDCHAR  ,SHIFT_F1,          -1,/*  0417  add and delete character */
LE_PCHAR   ,F2,                -1,/*  0420  recall character buffer */
LE_CCCHR   ,SHIFT_F2,          -1,/*  0421  abandon character buffer */
LE_CWORD   ,Home, F1,          -1,/*  0422  add word to character buffer */
LE_CDWORD  ,Home, SHIFT_F1,    -1,/*  0423  add and delete word */
LE_DWORD   ,Home, CTRL_A, RIGHT, -1,/*  0424  delete word */
LE_DLWORD  ,Home, CTRL_A, LEFT,  -1,/*  0425  delete previous word */
LE_CENTR   ,CTRL_T, CTRL_L,    -1,/*  0426  center line */

LE_RINS    ,ESC, Insert,       -1,/*  0427  set insert mode */
LE_RREP    ,ESC, Delete,       -1,/*  0430  set replace mode */
LE_CCUP    ,F10, UP,           -1,/*  0431  turn character up */
LE_CCDWN   ,F10, DOWN,         -1,/*  0432  turn character down */
LE_CCRUS   ,F10, RIGHT,        -1,/*  0433  turn character russian */
LE_CCLAT   ,F10, LEFT,         -1,/*  0434  turn character latin */
LE_CATTR   ,F10, F10,          -1,/*  0435  set character into current attributes */
LE_SPCHAR  ,CTRL_X,            -1,/*  0436  put character with given code */
LE_ESCCHR  ,                   -1,/*  0437  put character ESC (033 ) */
LE_HFCHAR  ,                   -1,/*  0440  put character HF */

LA_NONE    ,F10,' ',           -1,/*  0441  clear attributes */
LA_LIGHT   ,F10,'S',           -1,/*  0442  change lightnesss */
LA_LIGHT   ,F10,'s',           -1,/*  0442                    */
LA_ITALIC  ,F10,'M',           -1,/*  0443  change shimmering */
LA_ITALIC  ,F10,'m',           -1,/*  0443                    */
LA_UNDERL  ,F10,'P',           -1,/*  0444  change underlying */
LA_UNDERL  ,F10,'p',           -1,/*  0444                    */
LA_BOLD    ,F10,'V',           -1,/*  0445  change boldness   */
LA_BOLD    ,F10,'v',           -1,/*  0445                    */
LA_SUPER   ,F10,'W',           -1,/*  0446  */
LA_SUPER   ,F10,'w',           -1,/*  0446  */
LA_SUBSCR  ,F10,'N',           -1,/*  0447  */
LA_SUBSCR  ,F10,'n',           -1,/*  0447  */
LA_INVERT  ,F10,'I',           -1,/*  0450  change inversity  */
LA_INVERT  ,F10,'i',           -1,/*  0450                    */

TE_UNDO    ,F8,                -1,/*  0451  undo previous editing */
TE_UNUNDO  ,SHIFT_F8,          -1,/*  0452  redo previous editing */
TE_UNUNDO  ,Home, F8,          -1,/*  0452                        */
TE_SUNDO   ,                   -1,/*  0453  */
TE_SUNUNDO ,                   -1,/*  0454  */

TE_SRO     ,Home,CTRL_T,       -1,/*  0455  set read-only mode  */
TE_SRW     ,Home,CTRL_E,       -1,/*  0456  set read-write mode */
TE_SWIGHT  ,Home,CTRL_W,       -1,/*  0457  set text width */

TE_SMARK   ,CTRL_6, ' '        -1,/*  0460  set marker */
TE_CMARK   ,CTRL_6, CTRL_6,    -1,/*  0461  swap cursor and marker */

TE_PTAB    ,ESC,  Tab,         -1,/*  0462  edit tabulation pattern */
TE_PFORM   ,ESC,  F9,          -1,/*  0463  edit format pattern */

TE_UP      ,UP,                -1,/*  0464  move up */
TE_DOWN    ,DOWN,              -1,/*  0465  move down */
TE_CUP     ,Home,UP,           -1,/*  0466  contiguous motion up */
TE_CDOWN   ,Home,DOWN,         -1,/*  0467  contiguous motion down */
TE_TBEG    ,CTRL_6, UP,        -1,/*  0470  jump to first line text */
TE_TEND    ,CTRL_6, DOWN,      -1,/*  0471  jump to last line */

TE_IL      ,Home, Insert,      -1,/*  0472  insert blank line */
TE_DL      ,Home, Delete,      -1,/*  0473  delete current line */
TE_CR      ,Enter,             -1,/*  0474  go to beginning of the next line */
TE_RCR     ,                   -1,/*  0475  go to beginning of the previous line */
TE_SENTR   ,Home, F7,          -1,/*  0476  enter search/replace pattern */
TE_SUP     ,SHIFT_F7,          -1,/*  0477  search up */
TE_SDOWN   ,F7,                -1,/*  0500  search down */
TE_CENTR   ,CTRL_E,            -1,/*  0501  set the middle of the window to current line */
TE_CLIN    ,F3,                -1,/*  0502  add line to buffer */
TE_CDLIN   ,SHIFT_F3,          -1,/*  0503  add and delete line */
TE_PLIN    ,F4,                -1,/*  0504  recall line buffer */
TE_CCLIN   ,SHIFT_F4,          -1,/*  0505  abandon line buffer */

TE_BLIN    ,                   -1,/*  0506  cut line with displacement */
TE_SLIN    ,                   -1,/*  0507  paste lines with displacement */
TE_NBLIN   ,                   -1,/*  0510  cut line without displacement */
TE_NSLIN   ,                   -1,/*  0511  paste lines without displacement */
TE_IPAGE   ,                   -1,/*  0512  set page height */
TE_NPAGE   ,PageDown,          -1,/*  0513  go to next page */
TE_PPAGE   ,PageUp,            -1,/*  0514  go to previous page */

TE_MCBLOCK ,CTRL_B, CTRL_B,    -1,/*  0515  mark begin of block/select block */
TE_CDBLOCK ,CTRL_B, CTRL_U,    -1,/*  0516  select and delete block */
TE_CCBLOCK ,CTRL_B, CTRL_D,    -1,/*  0517  select and erase block */
TE_PBLOCK  ,CTRL_B, RIGHT,     -1,/*  0520  put block */
TE_PRBLOCK ,CTRL_B, LEFT,      -1,/*  0521  put block in back direction */
TE_PIBLOCK ,CTRL_B, UP,        -1,/*  0522  insert block */
TE_PIRBLOCK,CTRL_B, DOWN,      -1,/*  0523  insert block in back direction */
TE_BLKEXEC ,CTRL_B, Enter,     -1,/* 01021  execute block */
TE_BLKMOD  ,CTRL_B, CTRL_J,    -1,/*  ????  modify block attributes */

TE_LPRINT  ,                   -1,/*  0524  */
TE_FPRINT  ,                   -1,/*  0525  */
TE_FORMAT  ,F9,                -1,/*  0526  format paragraph in natural boundary */
TE_PFORMAT ,SHIFT_F9,          -1,/*  0527  format paragraph with your pattern */
TE_LDUP    ,ALT_D, UP,         -1,/*  0530  */
TE_LDDOWN  ,ALT_D, DOWN,       -1,/*  0531  */
TE_LDLEFT  ,ALT_D, LEFT,       -1,/*  0532  */
TE_LDRIGHT ,ALT_D, RIGHT,      -1,/*  0533  */
TE_RUP     ,ESC, SHIFT_F7,     -1,/*  0534  replace up */
TE_RDOWN   ,ESC, F7,           -1,/*  0535  replace down */

TE_PPRINT  ,                   -1,/*  0536  */
TE_IPRINT  ,                   -1,/*  0537  */
TE_RSPELL  ,                   -1,/*  0540  */
TE_LSPELL  ,                   -1,/*  0541  */
TE_SPADD   ,                   -1,/*  0542  */
TE_SPREM   ,                   -1,/*  0543  */

TE_HELP    ,ESC ,'/',          -1,/*  0544  help */
TE_HELP    ,ESC ,'?',          -1,/*  0544  help */
TE_OHELP   ,ESC ,'H',          -1,/*  0544  help */
TE_OHELP   ,ESC ,'h',          -1,/*  0544  help */
TE_MSGS    ,ESC ,'!',          -1,/* 01013  notify the Creator */
TE_FENTR   ,ESC ,DOWN,         -1,/*  0545  enter so named file */
TE_F2ENTR  ,ESC ,ESC ,         -1,/*  0546  enter named file */
TE_FEXEC   ,ESC, CTRL_J,       -1,/*  0547  */
TE_EXEC    ,Home, CTRL_J,      -1,/*  0547  */
TE_SHELL   ,CTRL_J             -1,/*  0550  execute temporary shell */
TE_INFILE  ,ESC,RIGHT,         -1,/*  0551  enter filename */
TE_OUTFILE ,ESC,LEFT,          -1,/*  0552  leave filename */

TE_MODFILE ,ESC,CTRL_U,        -1,/*  ????  modify file attributes */

TE_UPDATE  ,Home, ESC,         -1,/*  0553  save file to disk */
TE_UPDATE  ,CTRL_T, CTRL_U,    -1,/*  0553                    */
TE_EXIT    ,ESC, UP,           -1,/*  0554  save file and exit */
TE_EXIT    ,CTRL_T, CTRL_T,    -1,/*  0554                     */
TE_EXOLD   ,ESC, CTRL_C,       -1,/*  0555  exit without saving */
TE_EXOLD   ,CTRL_T, CTRL_C,    -1,/*  0555                      */
TE_EXBACK  ,ESC, BS,           -1,/*  0556  keep old and create new version */
TE_EXBACK  ,CTRL_T, CTRL_B,    -1,/*  0556                                  */
TE_QUIT    ,CTRL_T, CTRL_A,    -1,/*  0557  abandon window`s editing */
TE_CRASH   ,                   -1,/*  0557  crash editor */
TE_NERROR  ,                   -1,/*  0560  switch to next error */
TE_PERROR  ,                   -1,/*  0561  switch to previous error */
TE_ERROFF  ,                   -1,/* 01022  close errors window */
TE_STATUS  ,SHIFT_F5,          -1,/* 01023  open/close status window */
TE_ZOOM    ,CTRL_W, CTRL_W,    -1,/* 01024  zoom window */
TE_UFORK   ,ALT_W, UP,         -1,/*  0562  divide window up */
TE_DFORK   ,ALT_W, DOWN,       -1,/* 01003  divide window down */
TE_RFORK   ,ALT_W, RIGHT,      -1,/* 01004  divide window right */
TE_LFORK   ,ALT_W, LEFT,       -1,/*  0563  divide window left */
TE_UWINDOW ,CTRL_W, UP,        -1,/*  0564  jump to upper window */
TE_DWINDOW ,CTRL_W, DOWN,      -1,/*  0565  jump to lower window */
TE_RWINDOW ,CTRL_W, RIGHT      -1,/*  0566  jump to right window */
TE_LWINDOW ,CTRL_W, LEFT,      -1,/*  0567  jump to left window */

TK_BREAK   ,CTRL_C,            -1,/*  0570  cancel operation */
TK_CTRLO   ,CTRL_Z,            -1,/*  0571  close/open eyes */
TK_REPEAT  ,Home, Home,        -1,/*  0573  repeat previous command */
TK_BRUS    ,                   -1,/*  0574  set big russian letters */
TK_SRUS    ,CTRL_O,            -1,/*  0575  set small russian letters */
TK_BLAT    ,                   -1,/*  0576  set big latin letters */
TK_SLAT    ,CTRL_N,            -1,/*  0577  set small latin letters */
TK_SM+0    ,                   -1,/*  0600  set macro 0 */
TK_EM+0    ,                   -1,/*  0700  execute macro 0 */
TK_SM+1    ,                   -1,/*  0601  */
TK_EM+1    ,                   -1,/*  0701  */
TK_SM+2    ,                   -1,/*  0602  */
TK_EM+2    ,                   -1,/*  0702  */
TK_SM+3    ,                   -1,/*  0603  */
TK_EM+3    ,                   -1,/*  0703  */
TK_SM+4    ,                   -1,/*  0604  */
TK_EM+4    ,                   -1,/*  0704  */
TK_SM+5    ,                   -1,/*  0605  */
TK_EM+5    ,                   -1,/*  0705  */
TK_SM+6    ,                   -1,/*  0606  */
TK_EM+6    ,                   -1,/*  0706  */
TK_SM+7    ,                   -1,/*  0607  */
TK_EM+7    ,                   -1,/*  0707  */
TK_SM+8    ,                   -1,/*  0610  */
TK_EM+8    ,                   -1,/*  0710  */
TK_SM+9    ,                   -1,/*  0611  */
TK_EM+9    ,                   -1,/*  0711  */
TK_SM+10   ,                   -1,/*  0612  */
TK_EM+10   ,                   -1,/*  0712  */
TK_SM+11   ,                   -1,/*  0613  */
TK_EM+11   ,                   -1,/*  0713  */

TD_SDEQ    ,                   -1,/* 01001  dump deqs */
TD_STXT    ,                   -1,/* 01002  dump texts */

TK_CSET    ,                   -1,/*  ????  set counter */
TK_CADD    ,                   -1,/*  ????  add to counter */
TK_CSUB    ,                   -1,/*  ????  substract from counter */
TK_CPUT    ,                   -1,/*  ????  print counter */
TK_DATE    ,Home, CTRL_D,      -1,/* 01005  print current date */
TK_TIME    ,Home, CTRL_V,      -1,/* 01006  print current time */
MC_SAVE    ,                   -1,/* 01007  save macros */
MC_LOAD    ,                   -1,/* 01010  load macros */
MC_2SAVE   ,                   -1,/* 01011  save macros to named file */
MC_2LOAD   ,                   -1,/* 01012  load macros from named file */

EX_EXEC    ,                   -1,/* 02030  */
EX_CEXEC   ,                   -1,/* 02031  */
EX_FEXEC   ,                   -1,/* 02032  */

MC_INCL    ,                   -1,/* 02000  */
MC_INCL    ,                   -1,/* 02000  */
MC_PROC    ,                   -1,/* 02000  */
MC_PROC    ,                   -1,/* 02000  */
MC_SUBR    ,                   -1,/* 02000  */
MC_SUBR    ,                   -1,/* 02000  */
MC_FUNC    ,                   -1,/* 02000  */
MC_FUNC    ,                   -1,/* 02000  */
MC_WHIL    ,                   -1,/* 02000  */
MC_WHIL    ,                   -1,/* 02000  */
MC_DO      ,                   -1,/* 02000  */
MC_DO      ,                   -1,/* 02000  */
MC_IF      ,                   -1,/* 02000  */
MC_IF      ,                   -1,/* 02000  */

DB_DUMP    ,                   -1,/* 02000  */
DB_TRACE   ,                   -1,/* 02000  */

-1};

short special_table[] = {

TK_ESC     ,ESC,               /*  0572  Esc */
-1};

#include "gentab.c"
