#include "cmn.h"
#include "kbvt52.h"

#define MAXTABS 10
#define MAXLENTAB 200
#define FILE_NAME "vt52.cm"

short origin_table[] = {

LE_RIGHT   ,RIGHT,             -1,/*  0401    */
LE_LEFT    ,LEFT,              -1,/*  0402    */
LE_DEOL    ,CTRL_D,            -1,/*  0403       */
LE_DBGOL   ,CTRL_U,            -1,/*  0404       */
LE_BEG     ,HOME,LEFT,         -1,/*  0405      */
LE_END     ,HOME,RIGHT,        -1,/*  0406      */
LE_IC      ,IC,                -1,/*  0407    */
LE_DC      ,DC,                -1,/*  0410      */
LE_BS      ,ZB,                -1,/*  0411  /    */
LE_TAB     ,GT,                -1,/*  0412       */
LE_LTAB    ,TAB,               -1,/*  0413      */
LE_NWORD   ,                   -1,/*  0414    */
LE_PWORD   ,                   -1,/*  0415    */
LE_CCHAR   ,STR,               -1,/*  0416      */
LE_CDCHAR  ,                   -1,/*  0417      */
LE_PCHAR   ,STS,               -1,/*  0420     */
LE_CCCHR   ,HOME,STS,          -1,/*  0421     */
LE_CWORD   ,HOME,STR,          -1,/*  0422    */
LE_CDWORD  ,                   -1,/*  0423      */
LE_DWORD   ,S1,                -1,/*  0424        */
LE_DLWORD  ,                   -1,/*  0425     */
LE_CENTR   ,CTRL_T,            -1,/*  0426    */

LE_RINS    ,HOME,IC,           -1,/*  0427     */
LE_RREP    ,HOME,DC,           -1,/*  0430     */
LE_CCUP    ,                   -1,/*  0431    ->  */
LE_CCDWN   ,                   -1,/*  0432    ->  */
LE_CCRUS   ,                   -1,/*  0433    ->  */
LE_CCLAT   ,                   -1,/*  0434    ->  */
LE_CATTR   ,                   -1,/*  0435    ->   */
LE_SPCHAR  ,                   -1,/*  0436     <> */
LE_ESCCHR  ,                   -1,/*  0437  ESC (033 ) */
LE_HFCHAR  ,                   -1,/*  0440    (0???) */

LA_NONE    ,                   -1,/*  0441    :=  */
LA_LIGHT   ,                   -1,/*  0442    ^=   */
LA_ITALIC  ,                   -1,/*  0443    ^=    */
LA_UNDERL  ,                   -1,/*  0444    ^=  */
LA_BOLD    ,                   -1,/*  0445    ^=   */
LA_SUPER   ,                   -1,/*  0446    ^=   */
LA_SUBSCR  ,                   -1,/*  0447    ^=   */
LA_INVERT  ,                   -1,/*  0450    ^=  */

TE_UNDO    ,HOME,PS,           -1,/*  0451   */
TE_UNUNDO  ,HOME,WK,           -1,/*  0452    */
TE_SUNDO   ,                   -1,/*  0453   */
TE_SUNUNDO ,                   -1,/*  0454    */

TE_SRO     ,HOME,CTRL_T,       -1,/*  0455      */
TE_SRW     ,HOME,CTRL_E,       -1,/*  0456      */
TE_SWIGHT  ,                   -1,/*  0457     */

TE_SMARK   ,HOME,GT,           -1,/*  0460    */
TE_CMARK   ,HOME,TAB,          -1,/*  0461      */

TE_PTAB    ,SEND,TAB,          -1,/*  0462     */
TE_PFORM   ,SEND,S1,           -1,/*  0463     */

TE_UP      ,UP,                -1,/*  0464    */
TE_DOWN    ,DOWN,              -1,/*  0465    */
TE_CUP     ,HOME,UP,           -1,/*  0466    */
TE_CDOWN   ,HOME,DOWN,         -1,/*  0467    */
TE_TBEG    ,HOME,C1,           -1,/*  0470     */
TE_TEND    ,HOME,C2,           -1,/*  0471     */

TE_IL      ,IL,                -1,/*  0472    */
TE_DL      ,DL,                -1,/*  0473    */
TE_CR      ,WK,                -1,/*  0474      */
TE_RCR     ,PS,                -1,/*  0475      */
TE_SENTR   ,HOME,PRS,          -1,/*  0476      .  */
TE_SUP     ,SEND,PRS,          -1,/*  0477    */
TE_SDOWN   ,PRS,               -1,/*  0500    */
TE_CENTR   ,CTRL_E,            -1,/*  0501       */
TE_CLIN    ,AR1,               -1,/*  0502      */
TE_CDLIN   ,HOME,AR1,          -1,/*  0503      */
TE_PLIN    ,SBR,               -1,/*  0504     */
TE_CCLIN   ,HOME,SBR,          -1,/*  0505     */
TE_BLIN    ,HOME,IL,           -1,/*  0506      */
TE_SLIN    ,SEND,IL,           -1,/*  0507     */
TE_NBLIN   ,HOME,DL,           -1,/*  0510    */
TE_NSLIN   ,SEND,DL,           -1,/*  0511     */
TE_IPAGE   ,                   -1,/*  0512     */
TE_NPAGE   ,                   -1,/*  0513     */
TE_PPAGE   ,                   -1,/*  0514     */

TE_MCBLOCK ,                   -1,/*  0515    /  */
TE_CDBLOCK ,                   -1,/*  0516      */
TE_CCBLOCK ,                   -1,/*  0517      */
TE_PBLOCK  ,                   -1,/*  0520    */
TE_PRBLOCK ,                   -1,/*  0521       */
TE_PIBLOCK ,                   -1,/*  0522    */
TE_PIRBLOCK,                   -1,/*  0523       */

TE_LPRINT  ,                   -1,/*  0524    */
TE_FPRINT  ,                   -1,/*  0525      */
TE_FORMAT  ,                   -1,/*  0526       */
TE_PFORMAT ,HOME,S1,           -1,/*  0527      */
TE_LDUP    ,                   -1,/*  0530     */
TE_LDDOWN  ,                   -1,/*  0531     */
TE_LDLEFT  ,                   -1,/*  0532     */
TE_LDRIGHT ,                   -1,/*  0533     */
TE_RUP     ,                   -1,/*  0534      */
TE_RDOWN   ,                   -1,/*  0535      */

TE_PPRINT  ,                   -1,/*  0536  c   */
TE_IPRINT  ,                   -1,/*  0537  (  */
TE_RSPELL  ,                   -1,/*  0540  /  */
TE_LSPELL  ,                   -1,/*  0541 */
TE_SPADD   ,                   -1,/*  0542  $    */
TE_SPREM   ,                   -1,/*  0543     */

TE_HELP    ,SEND,'/',          -1,/*  0544  help */
TE_HELP    ,SEND,'?',          -1,/*  0544  help */
TE_MSGS    ,SEND,'!',          -1,/* 01013     */
TE_FENTR   ,SEND,DOWN,         -1,/*  0545      */
TE_F2ENTR  ,SEND,SEND,         -1,/*  0546         */
TE_FEXEC   ,SEND,PS,           -1,/*  0547    */
TE_SHELL   ,SEND,WK,           -1,/*  0550      cli */
TE_INFILE  ,SEND,RIGHT,        -1,/*  0551  enter filename */
TE_OUTFILE ,SEND,LEFT,         -1,/*  0552  leave filename */

TE_UPDATE  ,HOME,SEND,         -1,/*  0553      */
TE_EXIT    ,SEND,UP,           -1,/*  0554       */
TE_EXOLD   ,SEND,CTRL_C,       -1,/*  0555       */
TE_EXBACK  ,SEND,ZB,           -1,/*  0556  ,      */
TE_QUIT    ,SEND,CTRL_Q,       -1,/*  0557      */
TE_NERROR  ,                   -1,/*  0560      */
TE_PERROR  ,                   -1,/*  0561     */
TE_UFORK   ,SEND,AR2,UP,       -1,/*  0562   fork     */
TE_DFORK   ,SEND,AR2,DOWN,     -1,/* 01003   fork      */
TE_RFORK   ,SEND,AR2,RIGHT,    -1,/* 01004     fork      */
TE_LFORK   ,SEND,AR2,LEFT,     -1,/*  0563     fork       */
TE_UWINDOW ,AR2,UP,            -1,/*  0564     */
TE_DWINDOW ,AR2,DOWN,          -1,/*  0565      */
TE_RWINDOW ,AR2,RIGHT,         -1,/*  0566      */
TE_LWINDOW ,AR2,LEFT,          -1,/*  0567       */

TK_BREAK   ,CTRL_C,            -1,/*  0570       */
TK_CTRLO   ,CTRL_Z,            -1,/*  0571  /    */
TK_REPEAT  ,HOME,HOME,         -1,/*  0573  /   */
TK_BRUS    ,AR2,NR,            -1,/*  0574  /   */
TK_SRUS    ,NR,                -1,/*  0575  a    */
TK_BLAT    ,AR2,WR,            -1,/*  0576  /   */
TK_SLAT    ,WR,                -1,/*  0577  a    */
TK_SM+0    ,HOME,PRM,          -1,/*  0600     0 */
TK_EM+0    ,PRM,               -1,/*  0700   /  0 */
TK_SM+1    ,HOME,PRD,          -1,/*  0600     0 */
TK_EM+1    ,PRD,               -1,/*  0700   /  0 */
TK_SM+2    ,COMMA,N0,          -1,/*  0600     0 */
TK_EM+2    ,N0,                -1,/*  0700   /  0 */
TK_SM+3    ,COMMA,N1,          -1,/*  0600     0 */
TK_EM+3    ,N1,                -1,/*  0700   /  0 */
TK_SM+4    ,COMMA,N2,          -1,/*  0600     0 */
TK_EM+4    ,N2,                -1,/*  0700   /  0 */
TK_SM+5    ,COMMA,N3,          -1,/*  0600     0 */
TK_EM+5    ,N3,                -1,/*  0700   /  0 */
TK_SM+6    ,COMMA,N4,          -1,/*  0600     0 */
TK_EM+6    ,N4,                -1,/*  0700   /  0 */
TK_SM+7    ,COMMA,N5,          -1,/*  0600     0 */
TK_EM+7    ,N5,                -1,/*  0700   /  0 */
TK_SM+8    ,COMMA,N6,          -1,/*  0600     0 */
TK_EM+8    ,N6,                -1,/*  0700   /  0 */
TK_SM+9    ,COMMA,N7,          -1,/*  0600     0 */
TK_EM+9    ,N7,                -1,/*  0700   /  0 */
TK_SM+10   ,COMMA,N8,          -1,/*  0600     0 */
TK_EM+10   ,N8,                -1,/*  0700   /  0 */
TK_SM+11   ,COMMA,N9,          -1,/*  0600     0 */
TK_EM+11   ,N9,                -1,/*  0700   /  0 */

TD_SDEQ    ,                   -1,/*  01001* /  */
TD_STXT    ,                   -1,/*  01002* /  */
TK_DATE    ,SEND,PRD,          -1,/*  01005  "   */
TK_TIME    ,SEND,PRM,          -1,/*  01006  "   */
MC_SAVE    ,COMMA,DC,          -1,/*  01007  "    */
MC_LOAD    ,COMMA,IC,          -1,/*  01010  '    */
MC_2SAVE   ,COMMA,COMMA,DC,    -1,/*  01011  "    */
MC_2LOAD   ,COMMA,COMMA,IC,    -1,/*  01012  '    */
-1};

short special_table[] = {

TK_ESC     ,AR2,               /*  0572  Esc */
TK_ESC     ,SEND,              /*  0572  Esc */
-1};

#include "gentab.c"
