/*
 *     nanoMir
 *
 *     "color.c"        Color support using ISO 6429 (ANSI) control sequences
 */
#include "nm.h"
#include "me.h"
#include "sc.h"
#include "kb.h"
#include "trm.h"
#include "ac.h"
#include "tv.h"
#include <signal.h>
#ifdef VMS
#include <iodef.h>
#include <errno.h>
#endif
#ifdef UNIX
#include <termio.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef COLOR

#ifdef COLOR_ONLY_FOR_LINUX
static color_p = 1;
#endif /* COLOR_ONLY_FOR_LINUX */

static char TextColorString[MAXLPAC];

static int ConvertTextColor (char * ColorName)
{
  if (!strcmp (ColorName, "black"))
    return 30;
  if (!strcmp (ColorName, "red"))
    return 31;
  if (!strcmp (ColorName, "green"))
    return 32;
  if (!strcmp (ColorName, "yellow"))
    return 33;
  if (!strcmp (ColorName, "blue"))
    return 34;
  if (!strcmp (ColorName, "magenta"))
    return 35;
  if (!strcmp (ColorName, "cyan"))
    return 36;
  if (!strcmp (ColorName, "white"))
    return 37;
  return -1;
}

void InitTermColors ()
{
  char * envStr;
  char * p;
  int foregroundID;
  int backgroundID;
  int modeID;

#ifdef COLOR_ONLY_FOR_LINUX
  envStr = getenv (NMTERM);
  if (envStr && !strcmp ("linux", envStr))
    color_p = 1;
  else
    color_p = 0;
#endif /* COLOR_ONLY_FOR_LINUX */

  envStr = getenv (MIMFOREGROUND);
  if (!envStr || ! strcmp (envStr, ""))
    {
      /*    */
      envStr = DEFAULT_MIMFOREGROUND;
    }
  /*  ""  */
  p = strstr (envStr, "light");
  if (p && p == envStr)
    {
      modeID = 1;
      envStr += strlen ("light");
    }
  else
    {
      modeID = 0;
    }
  foregroundID = ConvertTextColor (envStr);
  if (foregroundID < 0)
    foregroundID = ConvertTextColor (DEFAULT_MIMFOREGROUND);

  envStr = getenv (MIMBACKGROUND);
  if (!envStr || ! strcmp (envStr, ""))
    {
      /*    */
      envStr = DEFAULT_MIMBACKGROUND;
    }
  backgroundID = ConvertTextColor (envStr);
  if (backgroundID < 0)
    backgroundID = ConvertTextColor (DEFAULT_MIMBACKGROUND);
  backgroundID += 10;
  sprintf (TextColorString, "\033[%2.2d;%2.2d;%2.2dm", 
           modeID, backgroundID, foregroundID);
}

static void
WriteStringToTerm (char *s)
{
  void (*p) (int);

#ifdef COLOR_ONLY_FOR_LINUX
  if (!color_p)
    return;
#endif /* COLOR_ONLY_FOR_LINUX */

  p = signal (SIGINT, SIG_IGN);
  write (term_file, s, strlen (s));
  ioctl (term_file, TCSBRK, 1);
  signal (SIGINT, p);
}

void
SetDefaultTextColor ()
{
  WriteStringToTerm ("\033[0m");
}

void
SetTextColor ()
{
#if 1
  WriteStringToTerm (TextColorString);
#else
  WriteStringToTerm ("\033[00;32m");
#endif
}
#endif /* ifdef COLOR */

#ifdef COLORE
void
SetEnterTextColor ()
{
  WriteStringToTerm ("\033[00;36m");
}
#endif /* ifdef COLORE */

#ifdef COLORB
void
SetBorderTextColor ()
{
  WriteStringToTerm ("\033[00;36m");
}
#endif /* ifdef COLORB */

#ifdef COLORS
void
SetStatusTextColor ()
{
  WriteStringToTerm ("\033[00;37m");
}
#endif /* ifdef COLORS */

#ifdef ALT_CHAR_SET_FONT
void
SetAltCharSetFont ()
{
  WriteStringToTerm ("\033(K");
}
#endif /* ALT_CHAR_SET_FONT */
