/*
 *     nanoMir
 *
 *     "er.c"           Process error and status line
 */

#include "nm.h"
#include "tm.h"
#include "cm.h"
#include "wd.h"
#ifndef X11
#include "sc.h"
#else
#include "xi.h"
#include "cmn.h"
#endif
#include "er.h"
#include "te.h"
#include "le.h"
#include "tx.h"
#include "fs.h"
#include "dq.h"
#include "ac.h"
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <string.h>

#define ERRFILE "errors.out"
#define MAXERR    50
#define MAXERRMSG 256
typedef struct
{
  char efile[MAXPATH];
  small eline;
  char emsg[MAXERRMSG];
}
errmsg;

#ifdef UNIX
local char *nm_errlist[] = {
  "syntax error",
  "no such drive",
  "no sush directory",
  "i/o error",
  "no such file",
  "directory not found"
};
#endif

static errmsg *erCe;

static wnd *stwnd;
bool stOpen;
bool erOpen;
bool erLoaded;
bool erHflag;
local errmsg errlist[MAXERR];
local int nerr, cerr;
local char status[NSTLINE + 1][MAXLPAC];
local tchar tline[MAXLPAC];

#if defined(X11) && defined (UNIX)
local tchar window_title[MAXLPAC];
#endif /* X11 && UNIX */

local void erLoad ();
local void erFix (register txt *, small);
local char *erPos ();
local void LoadBin (char *file, register deq **, bool teflag);
local void stView ();
local void stCreateWnd (small height,
			tchar * (*winf) (wnd *, small, small *));
local void stExtendWnd (small amount);
local stDestroyWnd ();
local void sttext (small y);
local tchar *stInfo (wnd * w, small y, small * pl);
local tchar *erInfo (wnd * w, small y, small * pl);

void
erInit ()
{
  register int i, j;

#if ( DBFS || DBFLUSH )
  fclose (stderr);
  stderr = *fopen ("nm.err", "w");
#endif

#if 0
  /*   "Segmentation fault" !!! */
  stOpen = TRUE;
#else
  stOpen = FALSE;
#endif
  erOpen = FALSE;
  erLoaded = FALSE;
  for (i = 0; i < NSTLINE + 1; i++)
    for (j = 0; j < MAXLPAC; j++)
      status[i][j] = ' ';
#ifndef STATUS_SHORT
#ifdef RUSSIAN
  scpy (":   ", status[0] + 23);
#endif
#ifdef ENGLISH
  scpy ("Attributes: ", status[0] + 23);
#endif
#endif /* STATUS_SHORT */
}

char *
erPrev ()
{
  erLoad ();

  if (!nerr)
    exc (E_FINDER);
  if (--cerr < 0)
    {
      cerr = 0;
      scbell ();
    }
  return (erPos ());
}

char *
erNext ()
{
  erLoad ();
  if (!nerr)
    exc (E_FINDER);
  if (++cerr >= nerr)
    {
      cerr = nerr - 1;
      scbell ();
    }
  return (erPos ());
}

void
erLoad ()
{
  register char *p, *p1;
  deq *d;

  if (erLoaded)
    return;
  erHflag = FALSE;
  nerr = 0;
  cerr = -1;
  erCe = errlist;
#ifdef NM
  d = NIL;
  LoadBin (ERRFILE, &d, TRUE);
  d->dtyp = DT_ASC;

  while (!qDqEmpt (d))
    {
      p = afbuf;
      *(p + DqGetB (d, p)) = 0;
      if (!strncmp (p, "Error ", 6))
	p += 6;
      else if (!strncmp (p, "Warning ", 8))
	p += 8;
      else
	continue;
      p1 = erCe->efile;
      while (*p != ' ')
	{
	  if (!*p)
	    continue;
	  *p1++ = *p++;
	}
      *p1 = 0;
      if (!tmcheck (erCe->efile))
	continue;
      p = dtos (p + 1, &(erCe->eline));
      erCe->eline--;
      if (*p != ':')
	continue;
      p1 = erCe->emsg;
      while (*p1++ = *p++);
      erCe++;
      nerr++;
      if (nerr == MAXERR)
	break;
    }
  DqDel (d);
  if (!nerr)
    exc (E_FINDER);
  erLoaded = TRUE;
#endif
  txILDL = erFix;
}

local char *
erPos ()
{
  register errmsg *e;
  register char *p;

  e = errlist + cerr;
  if (!(p = tmcheck (e->efile)))
    exc (E_FINDER);
  if (!scmp (Ttxt->finame, p))
    {
      if (erHflag)
	dspop ();
      dspush (e->efile);
      erHflag = TRUE;
      stCFile (Twnd->wtext->finame);
    }
  TxSetY (Ttxt, (large) (e->eline));
  Ty = Ttxt->txy;
  return (e->emsg);
}

local void
erFix (register txt * t, small d)
{
  register errmsg *e;

  if (!erLoaded)
    return;
  for (e = errlist; e < errlist + nerr; e++)
    if (scmp (t->finame, tmcheck (e->efile)) && t->txy <= e->eline)
      e->eline += d;
}

local void
LoadBin (char *file, register deq ** pd, bool teflag)
{
  register fid f;
  register deq *d;
  local txt er_txt;
  register txt *t = &er_txt;

  if (*pd != NIL)
    return;
  if (!FsParse (file, t->finame))
    goto err;
  if (!FsInfo (t))
    goto err;
  if (!(f = FsOpen (t->finame, FO_READ)))
    goto err;
  *pd = d = DqNew (DT_NONE, 0, 0);
  if (!DqLoad (d, f, t->fisize))
    goto err;
  if (!FsClose (f))
    {
    err:if ((d = *pd) != NIL)
	{
	  DqDel (d);
	  *pd = NIL;
	}
      if (teflag)
	ErFerr (t->finame);
      exc (E_PRINT);
    }
}

static struct errf
{
  small len;
  char *s[16];
  tchar a[16];
  small x[16], dx[16];
}
e;

#define DX 0
#define DY 0
#define MY 16
#ifdef X11
local tchar *
erWinInfo (wnd * w, small y, small * pl)
{
  register int i;
  if (++y > e.len)
    {
      *pl = 0;
      return tline;
    }
  for (i = 0; i < e.x[y]; i++)
    tline[i] = ctotc (' ');
  *pl = aftotc (e.s[y], e.dx[y], tline + e.x[y]);
  for (i = e.x[y] + e.dx[y]; i < MAXLPAC; i++)
    tline[i] = ctotc (' ');
  *pl += e.x[y];
  return tline;
}

local wnd *erwnd;
local void
erCreateWnd (int x, int y, int sw, int sh)
{
  register wnd *w;
  w = newWindow (x, y, sw, sh, WND_ERROR, 2);
  w->wstat |= WS_ERROR | WS_DEBUG;
  w->wtext = NIL;
  w->winf = erWinInfo;
  erwnd = w;
}

erDestroyWnd ()
{
  if (!(erwnd->wstat & WS_BUSY))
    return;
  WdFree (erwnd);
}

local void
ErIso ()
{
  int x, y, dx = 0, dy, scx, scy, i;
  if (tmNoText)
    printf ("%s: %s\r\n%s %s\r\n", e.s[2], e.s[3], e.s[5], e.s[6]);
  else
    {
      if (e.len > MY - 2 * DY)
	return;
      if (e.len < 2)
	return;
      scx = Twnd->wsw;
      scy = Twnd->wsh;
      for (i = 0; i < e.len; i++)
	{
	  e.dx[i] = slen (e.s[i]);
	  if (e.dx[i] > dx)
	    dx = e.dx[i];
	}
      dx += 4 + 2 * DX;
      dy = e.len + 2 * DY;
      for (i = 0; i < e.len; i++)
	e.x[i] = (dx - e.dx[i]) / 2;
      x = max (0, (scx - dx) / 2);
      y = max (0, (scy - dy) / 2);
      xiTranslateCoord (Twnd, &x, &y);
      erCreateWnd (x, y, dx, dy);
      while (KbGet () == TE_FOCUS);
      erDestroyWnd ();
    }
}
#else
local tchar buf[MY][MAXLPAC];
local void
ErIso ()
{
  small x, y, dx = 0, dy, mx, i, j;

  if (tmNoText)
    {
      tv_flush ();
      fprintf (stderr, "%s: %s\r\n%s %s\r\n", e.s[2], e.s[3], e.s[5], e.s[6]);
    }
  else
    {
      if (e.len > MY - 2 * DY)
	return;
      if (e.len < 2)
	return;
      mx = (scx * 5) / 6;
      for (i = 0; i < e.len; i++)
	{
	  e.dx[i] = slen (e.s[i]);
	  if (e.dx[i] > mx - 4 - 2 * DX)
	    e.dx[i] = mx - 4 - 2 * DX;
	  e.x[i] = (scx - e.dx[i]) / 2;
	  if (e.dx[i] > dx)
	    dx = e.dx[i];
	}
      dx += 4 + 2 * DX;
      dy = e.len + 2 * DY;
      x = (scx - dx) / 2;
      y = (scy - dy) / 2;
      ScSave (0);
      for (i = 0; i < dy; i++)
	for (j = 0; j < dx; j++)
	  buf[i][j] = ctotc (' ');
      buf[DY][DX] = ctotc (ldctoasc[LD_DOWN | LD_RIGHT]);
      buf[DY][dx - 1 - DX] = ctotc (ldctoasc[LD_DOWN | LD_LEFT]);
      buf[dy - 1 - DY][DX] = ctotc (ldctoasc[LD_UP | LD_RIGHT]);
      buf[dy - 1 - DY][dx - 1 - DX] = ctotc (ldctoasc[LD_UP | LD_LEFT]);
      for (i = 1 + DX; i < dx - 1 - DX; i++)
	buf[DY][i] = buf[dy - 1 - DY][i] =
	  ctotc (ldctoasc[LD_LEFT | LD_RIGHT]);
      for (i = 1 + DY; i < dy - 1 - DY; i++)
	buf[i][DX] = buf[i][dx - 1 - DX] = ctotc (ldctoasc[LD_UP | LD_DOWN]);
      for (i = 0; i < e.len; i++)
	for (j = 0; j < e.dx[i]; j++)
	  buf[DY + i][e.x[i] - x + j] = ctotc (e.s[i][j]) | (e.a[i]);
      for (i = 0; i < dy; i++)
	sctext (buf[i], dx, x, y + i);
      scpos (e.x[e.len - 1] + e.dx[e.len - 1], y + e.len - 1 + DY);
      tv_flush ();
      KbGet ();
      ScLoad (0);
    }
}
#endif

void
ErLert (char *p)
{
  e.len = 9;
  e.s[0] = e.s[1] = e.s[4] = e.s[5] = e.s[6] = e.s[7] = "";
#ifdef RUSSIAN
  e.s[2] = "";
#endif
#ifdef ENGLISH
  e.s[2] = "Error";
#endif
  e.a[2] = AT_INVERT | AT_BOLD;
  e.s[3] = p;
  e.a[3] = 0;
#ifdef RUSSIAN
  e.s[8] = "    ...";
#endif
#ifdef ENGLISH
  e.s[8] = " Press any key ...";
#endif
  e.a[8] = AT_BOLD;
  ErIso ();
}

void
ErFerr (char *file)
{
  char buf[1024];
  register char *p;
  if (!errno)
    return;
  e.len = 9;
  (e.s[0]) = (e.s[1]) = (e.s[4]) = (e.s[7]) = "";
#ifdef RUSSIAN
  e.s[2] = "";
#endif
#ifdef ENGLISH
  e.s[2] = "Error";
#endif
  e.a[2] = AT_INVERT;
#ifdef VMS
  e.s[3] = strerror (errno, vaxc$errno);
#endif
#ifdef UNIX
  if (errno > 0)
    e.s[3] = (char *) strerror (errno);
  else
    e.s[3] = nm_errlist[(-errno) - FE_MIN];
#endif
  e.a[3] = 0;
#ifdef RUSSIAN
  e.s[5] = " ";
#endif
#ifdef ENGLISH
  e.s[5] = "in file";
#endif
  e.a[5] = 0;
  e.s[6] = file;
  e.a[6] = 0;
#ifdef RUSSIAN
  e.s[8] = "    ...";
#endif
#ifdef ENGLISH
  e.s[8] = " Press any key ...";
#endif
  e.a[8] = AT_BOLD;
  ErIso ();
}

void
ErShow (register char *p)
{
  register wnd *w;
  register tchar *t;
  register small i;

  if (!erOpen)
    if (stOpen)
      {
	stwnd->winf = stInfo;
	stExtendWnd (1);
      }
    else
      stCreateWnd (1, erInfo);
  erOpen = TRUE;
  for (i = 0, t = tcbuf; i < MAXLPAC; i++)
    {
      if (*p)
	*t++ = ((*p++) & 0377) | AT_INVERT;
      else
	*t++ = ((tchar) ' ') | AT_INVERT;
    }
  tctoaf (tcbuf, MAXLPAC - 1, status[NSTLINE]);
  stView ();
}

void
stOn ()
{
#ifdef X11
  if (stOpen)
    return;
  stCreateWnd (NSTLINE, stInfo);
  stwnd->winf = stInfo;
  stOpen = TRUE;
#else
  if (stOpen)
    return;
  if (erOpen)
    {
      stwnd->winf = stInfo;
      stExtendWnd (NSTLINE);
    }
  else
    stCreateWnd (NSTLINE, stInfo);
  stwnd->winf = stInfo;
  stOpen = TRUE;
#endif
}

#ifdef X11
local void
stView ()
{
  if (!stOpen)
    return;
  wredraw (stwnd);
}
#else
local void
stView ()
{
  int x, y;
  if (!stOpen && !erOpen)
    return;
  x = cx;
  y = cy;
  wredraw (stwnd);
  scpos (x, y);
  tv_flush ();
}
#endif

#ifdef X11
local void
sttext (small y)
{
  small l;
  tchar *tp;

  if (!stOpen)
    return;
  tp = (*stwnd->winf) (stwnd, y, &l);
#ifdef STATUS_IN_MAIN_WINDOW
  {
  Window wxwin = stwnd->wxwin;
  Window wxmain = stwnd->wxmain;

  stwnd->wxwin = Twnd->wxwin;
  stwnd->wxmain = Twnd->wxstatusbar;

  if (stwnd->wxmain)
    {
      l = min (l, Twnd->wsw);
      xiText (stwnd, 0, y, tp, l);
    }

  stwnd->wxwin = wxwin;
  stwnd->wxmain = wxmain;
  }
#else
  l = min (l, stwnd->wsw);
  xiText (stwnd, 0, y, tp, l);
#endif /* STATUS_IN_MAIN_WINDOW */
}
#else
local void
sttext (small y)
{
  small ox = cx, oy = cy;
  small l;
  tchar *tp;

  if (!stOpen)
    return;
  tp = (*stwnd->winf) (stwnd, y, &l);
  l = min (l, stwnd->wsw);
  sctext (tp, l, stwnd->wsx, stwnd->wsy + y);
  scpos (ox, oy);
  tv_flush ();
}
#endif /* X11 */

void
stCFile (char *file)
{
  char *p;
  char *basename;

#ifdef STATUS_SHORT
  /* ,       */
  int wlen = (stwnd ? stwnd->wsw : 80) - STATUS_FILE_OFFSET;
  int flen = strlen (file);
  if (wlen >= flen)
    {
      /*   */
      p = status[0] + STATUS_FILE_OFFSET;
      p = scpy (file, p);
      /*   */
      for (; p < status[1] - 1; *p++ = ' ');
    }
  else
    {
      /*   */
      p = status[0] + STATUS_FILE_OFFSET;
      p = scpy ("...", p);
      p = scpy (file + flen - wlen + 3, p);
    }
  sttext(0);
#else /* ! STATUS_SHORT */

#ifdef RUSSIAN
  p = scpy ("  : ", status[NSTLINE - 1]);
#endif
#ifdef ENGLISH
  p = scpy ("  File: ", status[NSTLINE - 1]);
#endif
  p = scpy (file, p);
  for (; p < status[NSTLINE] - 1; *p++ = ' ');
  stView ();
#endif /* STATUS_SHORT */

#ifdef UNIX
  /*       */
  basename = strrchr (file, '/');

  if (!basename)
    basename = file;
  else
    basename++;

#ifndef REPLACE_SPACES_BY_TABS
  /*      */
  if (strcmp (basename, "Makefile") == 0 || strcmp (basename, "makefile") == 0)
    TX_ReplaceSpacesByTabs = 1;
  else
    TX_ReplaceSpacesByTabs = 0;
#endif /* REPLACE_SPACES_BY_TABS */
#endif /* UNIX */

#ifdef X11
#ifdef UNIX
  {
    /*    :    micros.dir */
    if (basename != file && strcmp (basename, ROOTFILE) == 0)
      {
	char dirname[MAXPATH];

	/*          */
	strcpy (dirname, file);
	dirname[basename - file - 1] = '\0';
	basename = strrchr (dirname, '/');
	if (!basename)
	  basename = dirname;
	else
	  basename++;
      }

    /*         
           */
    xiChangeIconName (Twnd, (char *) basename);
    p = scpy ("mim: ", (char *) window_title);
    p = scpy (basename, p);
    *p = '\0';
    xiChangeName (Twnd, (char *) window_title);
  }
#else
  xiChangeName (Twnd, file);
#endif /* UNIX */
#endif /* X11 */
}

#ifdef STATUS_SHORT
void
stCLines (large n)
{
  char s[16];

  sprintf (s, "%6d", n);
  scpy (s, status[0] + STATUS_LENGTH_OFFSET);

  sttext (0);
}
#else /* ! STATUS_SHORT */
void
stCLines (large n)
{
}
#endif /* STATUS_SHORT */

#ifdef STATUS_SHORT
#ifdef RUSSIAN
static char caps[4][2] = { "L", "l", "", "" };
#endif
#ifdef ENGLISH
static char caps[4][2] = { "L", "l", "N", "n" };
#endif
void
stCCaps (small n)
{
  scpy (caps[n], status[0] + STATUS_LANG_OFFSET);
  sttext (0);
}

#else /* ! STATUS_SHORT */

#ifdef RUSSIAN
static char caps[4][4] = { "LAT", "lat", "", "" };
#endif
#ifdef ENGLISH
static char caps[4][4] = { "LAT", "lat", "NAT", "nat" };
#endif
void
stCCaps (small n)
{
  scpy (caps[n], status[0] + 6);
  stView ();
}
#endif /* STATUS_SHORT */

#ifdef STATUS_SHORT
#ifdef RUSSIAN
static char mode[2][2] = { "", "" };
#endif
#ifdef ENGLISH
static char mode[2][2] = { "r", "i" };
#endif
void
stCMode (small n)
{
  scpy (mode[n], status[0] + STATUS_MODE_OFFSET);
  sttext (0);
}

#else /* ! STATUS_SHORT */

#ifdef RUSSIAN
static char mode[2][8] = { " ", "", };
#endif
#ifdef ENGLISH
static char mode[2][8] = { "Replace", "Insert ", };
#endif
void
stCMode (small n)
{
  scpy (mode[n], status[0] + 12);
  stView ();
}
#endif /* STATUS_SHORT */

#ifdef STATUS_SHORT
#ifdef RUSSIAN
local char modif[2][2] = { "", " " };
#endif
#ifdef ENGLISH
local char modif[2][2] = { "M", " " };
#endif
void
stModification (small n)
{
  scpy (modif[n], status[0] + STATUS_MODIF_OFFSET);
  sttext (0);
}

#else /* ! STATUS_SHORT */

#ifdef RUSSIAN
local char modif[2][4] = { "", "   ", };
#endif
#ifdef ENGLISH
local char modif[2][4] = { "Mod", "   ", };
#endif
void
stModification (small n)
{
  scpy (modif[n], status[0] + 44);
  stView ();
}
#endif /* STATUS_SHORT */

/*
     (   ,   ac.h )
  AC_LIGHT    001         
  AC_ITALIC   002        
  AC_UNDERL   004        
  AC_BOLD     010        
  AC_SUBSTR   020         
  AC_SUPER    040         
  AC_INVERT   0100        
  AC_BLOCK    0200       
*/

#ifdef STATUS_SHORT
#ifdef RUSSIAN
local char atname[][5] = { "", "", "", "", "", "", "", "" };
#endif
#ifdef ENGLISH
local char atname[][5] = { "l", "i", "u", "b", "s", "p", "k", "m" };
#endif
void
stCAttr (tchar attr)
{
  small i;
  tchar t;

#ifdef  STATUS_SHORT_ATTRIB
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    if (attr & t)
      break;
  scpy ((i < 8) ? "*" : " ", status[0] + STATUS_ATTRIB_OFFSET);
#else
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    scpy ((attr & t) ? atname[i] : " ", status[0] + STATUS_ATTRIB_OFFSET + i);
#endif /* STATUS_SHORT_ATTRIB */
  sttext (0);
}

#else /* ! STATUS_SHORT */

#ifdef RUSSIAN
local char atname[][5] = { "", "", "", "", "", "", "", "" };
#endif
#ifdef ENGLISH
local char atname[][5] = { "l", "b", "u", "o", "s", "p", "i", "k" };
#endif
local small atpos[] = { 35, 36, 37, 38, 39, 40, 41, 42 };
void
stCAttr (tchar attr)
{
  small i;
  tchar t;
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    if (attr & t)
      scpy (atname[i], status[0] + atpos[i]);
    else
      scpy (" ", status[0] + atpos[i]);
  stView ();
}
#endif /* STATUS_SHORT */

#ifdef STATUS_SHORT
void
stCXYSym (small x, large y)
{
  small i;
  tchar t;
#ifndef X11
  tchar sym;
#endif
  char s[64];

#ifdef STATUS_IN_MAIN_WINDOW
  /*     */
  Twnd->wcx = x;
  Twnd->wcy = y;
#endif /* STATUS_IN_MAIN_WINDOW */

  /* Positions are numbered starting from 1. */
  x++;
  y++;

#ifdef X11
  sprintf (s, "(%3d,%6d)", x & 0377, y);
#else
  sym = scim (cx, cy);
  sprintf (s, "(%3d,%6d,%3x)", x & 0377, y, sym & 0377);
#endif

  scpy (s, status[0] + STATUS_XY_OFFSET);

#ifndef X11
  sym &= ~0377;
#ifdef  STATUS_SHORT_ATTRIB
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    if (sym & t)
      break;
  scpy ((i < 8) ? "*" : " ", status[0] + STATUS_ATTRIB_OFFSET);
#else
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    scpy ((sym & t) ? atname[i] : " ", status[0] + STATUS_ATTRIB_OFFSET + i);
#endif /* STATUS_SHORT_ATTRIB */
#endif

  sttext (0);
}

#else /* ! STATUS_SHORT */

void
stCXYSym (small x, large y)
{
  small i;
  tchar t;
  tchar sym;
  char s[64];

  /* Positions are numbered starting from 1. */
  x++;
  y++;

#ifndef X11
  sym = scim (cx, cy);
#endif
#ifdef RUSSIAN
#ifdef X11
  sprintf (s, "X=%-3d Y=%-8d", x & 0377, y);
#else
  sprintf (s, "X=%-3d Y=%-8d    : %02x", x & 0377, y, sym & 0377);
#endif
#endif
#ifdef ENGLISH
#ifdef X11
  sprintf (s, "X=%-3d Y=%-8d", x & 0377, y);
#else
  sprintf (s, "X=%-3d Y=%-8d   code: %02x", x & 0377, y, sym & 0377);
#endif
#endif
  scpy (s, status[1] + 3);
  sym &= ~0377;
#ifndef X11
  for (i = 0, t = AT_LIGHT; i < 8; t <<= 1, i++)
    if (sym & t)
      scpy (atname[i], status[1] + atpos[i]);
    else
      scpy ("    ", status[1] + atpos[i]);
#endif
  sttext (1);
}
#endif /* STATUS_SHORT */

#ifdef STATUS_SHORT
static char zm[2][2] = { " ", "Z" };
void
stCZoom (small n)
{
  scpy (zm[n], status[0] + STATUS_ZOOM_OFFSET);
  sttext (0);
}

#else /* ! STATUS_SHORT */

static char zm[2][5] = { "    ", "ZOOM" };
void
stCZoom (small n)
{
  scpy (zm[n], status[0]);
  stView ();
}
#endif /* STATUS_SHORT */

void
stOff ()
{
#ifdef X11
  if (!stOpen)
    return;
  stDestroyWnd ();
  stOpen = FALSE;
#else
  register wnd *w;
  register small i;

  if (!stOpen)
    return;
  if (erOpen)
    {
      stwnd->winf = erInfo;
      stExtendWnd (-NSTLINE);
    }
  else
    stDestroyWnd ();
  stOpen = FALSE;
#endif
}

void
ErOff ()
{
  register wnd *w;

  if (!erOpen)
    return;
  if (stOpen)
    stExtendWnd (-1);
  else
    stDestroyWnd ();
  erOpen = FALSE;
}

#ifdef X11
local void
stCreateWnd (small height, tchar * (*winf) (wnd *, small, small *))
{
  register wnd *w;
  w = newWindow (xiSTX (), xiSTY (), 74, height, WND_STATUS, 1);
  w->wstat |= WS_ERROR | WS_DEBUG;
  w->wtext = NIL;
  w->winf = winf;
  stwnd = w;
  /* xiChangeName( w, "Status" ); */
}
#else
local void
stCreateWnd (small height, tchar * (*winf) (wnd *, small, small *))
{
  register small i, j;
  register wnd *w;

  for (i = MAXWNDWS - 1, w = windows; i; i--, w++)
    if (((j = w->wstat) & WS_BUSY) && !(j & WS_NOTEXT) && !(w->wsy))
      {
	wdx = 0;
	wdy = height + 1;
	wxmin = wymin = 0;
	wxmax = w->wsw;
	wymax = w->wsh;
	wroll (w);
	w->wsy = height + 1;
	w->wsh -= height + 1;
      }
  if (tmQZoom () && !zwsy)
    {
      zwsy = height + 1;
      zwsh -= height + 1;
    }
  w = newWindow (0, 0, scx, height, WND_SPECIAL, 1);
  w->wstat |= WS_ERROR | WS_DEBUG;
  w->wtext = NIL;
  w->winf = winf;
  wredraw (w);
  stwnd = w;
  WdShowboxes ();
}
#endif

#ifdef X11
local void
stExtendWnd (small amount)
{
}
#else
local void
stExtendWnd (small amount)
{
  register small i, j;
  register wnd *w;
  small y;

  if (!(stwnd->wstat & WS_BUSY))
    return;
  y = stwnd->wsy + stwnd->wsh + 1;
  for (i = MAXWNDWS - 1, w = windows; i; i--, w++)
    if (((j = w->wstat) & WS_BUSY) && !(j & WS_NOTEXT) && w->wsy == y)
      {
	w->wsy += amount;
	w->wsh -= amount;
	wdx = 0;
	wdy = amount;
	wxmin = wymin = 0;
	wxmax = w->wsw;
	wymax = w->wsh;
	wroll (w);
      }
  if (tmQZoom () && zwsy == y)
    {
      zwsy += amount;
      zwsh -= amount;
    }
  stwnd->wsh += amount;
  wredraw (stwnd);
  WdShowboxes ();
}
#endif

#ifdef X11
local
stDestroyWnd ()
{
  if (!(stwnd->wstat & WS_BUSY))
    return;
  WdFree (stwnd);
}
#else
local
stDestroyWnd ()
{
  register small i, j;
  register wnd *w;
  small y;

  if (!(stwnd->wstat & WS_BUSY))
    return;
  y = stwnd->wsy + stwnd->wsh + 1;
  for (i = MAXWNDWS - 1, w = windows; i; i--, w++)
    if (((j = w->wstat) & WS_BUSY) && !(j & WS_NOTEXT) && w->wsy == y)
      {
	w->wsy = 0;
	w->wsh -= (wdy = -y);
	wdx = 0;
	wxmin = wymin = 0;
	wxmax = w->wsw;
	wymax = w->wsh;
	wroll (w);
      }
  if (tmQZoom () && zwsy == y)
    {
      zwsy = 0;
      zwsh += y;
    }
  WdFree (stwnd);
  WdShowboxes ();
}
#endif

local tchar *
stInfo (wnd * w, small y, small * pl)
{
  *pl = aftotc (status[y], MAXLPAC, tline);
  return tline;
}

local tchar *
erInfo (wnd * w, small y, small * pl)
{
  if (y > 0)
    otkaz ("ER");
  *pl = aftotc (status[NSTLINE], MAXLPAC, tline);
  return tline;
}


#ifdef X11
void
setStatusWidth (int width)
{
  if (stwnd)
    stwnd->wsw = width;
}

void
redrawStatusBar ()
{
#ifdef STATUS_SHORT
  sttext (0);
#else /* ! STATUS_SHORT */
  stView ();
#endif /* STATUS_SHORT */
}

#ifdef STATUS_IN_MAIN_WINDOW
/*      W */
void
redrawWndStatusBar (wnd * w)
{
  Window wxwin = stwnd->wxwin;
  Window wxmain = stwnd->wxmain;

  if (!stOpen)
    return;

  stwnd->wxwin = w->wxwin;
  stwnd->wxmain = w->wxstatusbar;

#ifdef STATUS_SHORT
  {
    char s[64];
    int i;
    char *p;

    /*     */
    int x = w->wcx + 1;
    int y = w->wcy + 1;
    char *file = w->wtext->finame;
    large n = w->wtext->flines;

    int wlen = w->wsw - STATUS_FILE_OFFSET;
    int flen = strlen (file);
    small l;

    if (w->wtyp != WND_NORMAL)
      return;

    for (i = 0; i < MAXLPAC - 1; i++)
      status[0][i] = status[0][i];
    status[0][i] = '\0';

    /*       :
          .
          ,  stwnd->winf   stInfo */

    /*  */
    sprintf (s, "(%3d,%6d)", x & 0377, y);
    scpy (s, status[0] + STATUS_XY_OFFSET);

    /*   */
    /* ,       */
    if (wlen >= flen)
      {
	/*   */
	p = status[0] +	STATUS_FILE_OFFSET;
	p = scpy (file, p);
	/*   */
	for (; p < status[0] + MAXLPAC - 1; *p++ = ' ');
      }
    else
      {
	/*   */
	p = status[0] +	STATUS_FILE_OFFSET;
	p = scpy ("...", p);
	p = scpy (file + flen - wlen + 3, p);
      }

    /*   */
    sprintf (s, "%6d", n);
    scpy (s, status[0] + STATUS_LENGTH_OFFSET);

    /*   */
    scpy (((w->wtext->txstat & TS_CHANGE) ? modif[0] : modif[1]),
	  status[0] + STATUS_MODIF_OFFSET);

    /*  Caps, Mode, Zoom     */

    /*    	status[0] */

    /*  Ascii   tchar */
    l =	aftotc (status[0], MAXLPAC, tline);
    l = min (l, w->wsw);
    xiText (stwnd, 0, 0, tline,	l);

#if 0				/* DEBUG */
    status[0][l] = '\0';
    printf ("%s\n", status[0]);
#endif
  }

#else /* ! STATUS_SHORT */

  stView ();
#endif /* STATUS_SHORT */

  stwnd->wxwin = wxwin;
  stwnd->wxmain = wxmain;
}

void
linkStatusBarAndCurrentWnd ()
{
  if (stwnd->wxwin != Twnd->wxwin || stwnd->wxmain != Twnd->wxstatusbar)
    {
      stwnd->wxwin = Twnd->wxwin;
      stwnd->wxmain = Twnd->wxstatusbar;
      stwnd->wsw = Twnd->wsw;
      redrawWndStatusBar (Twnd);
    }
}

#endif /* STATUS_IN_MAIN_WINDOW */

#endif /* X11 */
