/*
 *     nanoMir
 *
 *     "kb.c"           Keyboard
 */

#include "nm.h"
#include "cm.h"
#include "cmn.h"
#include "kb.h"
#include "te.h"
#include "er.h"
#ifndef X11
#include "sc.h"
#include "tv.h"
#else
#include "xi.h"
#endif
#ifdef UNIX
#include <fcntl.h>
#endif
#ifdef VMS
#include <file.h>
#endif

large KbCount;
small KbRadix;
kchar KbCode;
bool kbBreak;
void (*KbWaitIn) ();

#define MS_NONE 0
#define MS_INP  1
#define MS_EXE  2

#define macro struct mstr
static struct mstr
{
  small mbeg;
  small mlen;
  large mctr;
  large moctr;
  small mptr;
  macro *mprev;
  small mstat;
}
 *macros;

static kchar *macbuf;
static macro *inmacro;
static macro *exmacro;
static small macslen;
static large oldcount;

local char mcbuf[MAXMCBUF];
local small mcbuflen;
local small mcbufptr;
local small nm_counter;

void
kbInit ()
{
  register small i;
  register macro *pm;

  macros = (macro *) GetMain (MACNOMAX * sizeof (macro));
  macbuf = (kchar *) GetMain (MACSIZMAX * sizeof (kchar));

  KbWaitIn = br14;
#ifdef X11
  xiAttach ();
#endif
  term_attach ();
  have_tty = TRUE;
  kbhcbrk ();
  for (i = MACNOMAX, pm = macros; i; i--, pm++)
    {
      pm->mbeg = 0;
      pm->mlen = 0;
      pm->mstat = MS_NONE;
    }
  inmacro = exmacro = NIL;
  macslen = 0;
  mcbuflen = 0;
  nm_counter = 0;
  KbCode = 0;
  KbCount = 0;
}

void
KbTerm ()
{
  kbDetach ();
}

kchar
KbQGet ()
{
  register long l;
  register kchar k;
  if (!qkbhin ())
    {
      (*KbWaitIn) ();
#ifndef X11
      scurson ();
#endif /* not def X11 */
    }
  k = prdcd ();
  qKbBreak ();
  return k;
}

kchar
qKbReq ()
{
  return (qkbhin ()? KbQGet () : 0);
}

bool
qKbBreak ()
{
  register small i;
  register macro *pm;
  if (qkbhbrk ())
    {
      kbhcbrk ();
      clear_tty_in ();
      for (i = MACNOMAX, pm = macros; i; i--, pm++)
	pm->mstat = MS_NONE;
      inmacro = exmacro = NIL;
      return (TRUE);
    }
  return (FALSE);
}

/* get char from queue/macro */
static kchar
get2 ()
{
  register kchar c;
  register macro *pm;

  if (mcbuflen)
    {
      if (mcbufptr < mcbuflen)
	{
	  c = (kchar) mcbuf[mcbufptr++];
	  return (c & 0377);
	}
      else
	mcbuflen = 0;
    }
nextc:
  if ((pm = exmacro) == NIL)
    {
      c = KbQGet ();
      if ((pm = inmacro) != NIL)
	{
	  if (macslen < MACSIZMAX)
	    {
	      macbuf[macslen++] = c;
	      pm->mlen++;
	    }
	  else
	    {
	      scbell ();
	      pm->mstat = MS_NONE;
	      inmacro = NIL;
	    }
	}
    }
  else
    {
      if (pm->mptr < pm->mlen)
	{
	  c = macbuf[pm->mbeg + pm->mptr];
	  pm->mptr++;
	}
      else if (--pm->mctr > 0)
	{
	  pm->mptr = 0;
	  goto nextc;
	}
      else
	{
	  exmacro = pm->mprev;
	  KbCode = TK_EM + (pm - macros);
	  oldcount = pm->moctr;
	  pm->mstat = MS_NONE;
	  goto nextc;
	}
    }
  return (c);
}

kchar
KbGet ()
{
  register kchar c;
  register small i;
  register macro *pm, *pm1;
nextc1:
  KbRadix = 0;
  KbCount = 1;
nextc:
  c = get2 ();
  switch (c)
    {
    case TK_DATE:
      sncpy (dttm (GetTime ()), mcbuf, 11);
      mcbuflen = 11;
      mcbufptr = 0;
      goto nextc;
    case TK_TIME:
      sncpy (dttm (GetTime ()) + 13, mcbuf, 8);
      mcbuflen = 8;
      mcbufptr = 0;
      goto nextc;
    case TK_CPUT:
      sprintf (mcbuf, "%6d", nm_counter);
      mcbuflen = 6;
      mcbufptr = 0;
      goto nextc;
    }
cchar:
  if (c >= TK_SM && c < (TK_SM + MACNOMAX))
    {
      if (exmacro != NIL)
	otkaz ("kbexmacro");
      if ((pm = inmacro) != NIL)
	{
	  pm->mlen--;
	  macslen--;
	  pm->mstat = MS_NONE;
	}
      inmacro = pm = macros + (c - TK_SM);
      i = pm->mbeg + pm->mlen;
      blkmov ((char *) (macbuf + i), (char *) (macbuf + pm->mbeg),
	      (macslen - i) * sizeof (kchar));

      for (pm1 = macros, i = MACNOMAX; i; pm1++, i--)
	if (pm1->mbeg > pm->mbeg)
	  pm1->mbeg -= pm->mlen;
      pm->mbeg = (macslen -= pm->mlen);
      pm->mlen = 0;
      pm->mstat = MS_INP;
      goto nextc1;
    }
  if (c >= TK_EM && c < TK_EM + MACNOMAX)
    {
      pm = macros + (c - TK_EM);
      if (pm->mstat == MS_INP)
	{
	  pm->mlen--;
	  macslen--;
	  pm->mstat = MS_NONE;
	  inmacro = NIL;
	}
      else
	{
	  if (pm->mstat != MS_NONE)
	    scbell ();
	  else if (i = KbCount)
	    {
	      pm->mprev = exmacro;
	      exmacro = pm;
	      pm->mstat = MS_EXE;
	      pm->mctr = pm->moctr = i;
	      pm->mptr = 0;
	    }
	}
      goto nextc1;
    }
  if (c == TK_ESC)
    {
      c = get2 ();
      if (c == '0')
	{
	  KbRadix = 8;
	  c = get2 ();
	  if ((c < 0400) && ((c & 0137) == 'X'))
	    {
	      KbRadix = 16;
	      c = get2 ();
	    }
	  goto digits;
	}
      if (c > '0' && c <= '9')
	{
	  KbRadix = 10;
	digits:
	  KbCount = 0;
	  for (;;)
	    {
	      if (c >= 0400)
		break;
	      i = (c > 0100 ? (c & 0137) - ('A' - 10) : c - '0');
	      if (i < 0 || i >= KbRadix)
		break;
	      KbCount = KbCount * KbRadix + i;
	      c = get2 ();
	    }
	}
      goto cchar;
    }
  switch (c)
    {
    case TK_REPEAT:
      KbCount *= oldcount;
      c = KbCode;
      goto cchar;
    case TK_CSET:
      nm_counter = KbCount;
      goto nextc1;
    case TK_CADD:
      nm_counter += KbCount;
      goto nextc1;
    case TK_CSUB:
      nm_counter -= KbCount;
      goto nextc1;
    }

  oldcount = KbCount;
  KbCode = c;
  return (KbCode);
}


local small macro_magic = 2301;

void
KbMSave (char *macro_file)
{
  register int fd;
  register small i;
  small j;
  register macro *pm;

#define twrite(s) write( fd, &(s), sizeof(s));

  if (macro_file == NIL || inmacro != NIL)
    {
      scbell ();
      return;
    }
  if (access (macro_file, 04) == 0)
    {
      fd = open (macro_file, O_RDONLY);
      read (fd, &j, sizeof (j));
      close (fd);
      if (j != macro_magic)
	{
	  scbell ();
	  return;
	}
    }
  fd = creat (macro_file, 0666);
  if (fd < 0)
    {
      scbell ();
      return;
    }
  twrite (macro_magic);
  j = MACNOMAX;
  twrite (j);
  j = MACSIZMAX;
  twrite (j);
  twrite (macslen);
  if (macslen)
    {
      for (i = MACNOMAX, pm = macros; i; pm++, i--)
	{
	  twrite (pm->mbeg);
	  twrite (pm->mlen);
	}
      write (fd, macbuf, macslen * sizeof (kchar));
    }
  close (fd);
}

void
KbMLoad (char *macro_file)
{
  register int fd;
  register small i;
  small j;
  register macro *pm;

#define tread(s) read( fd, &(s), sizeof(s));

  if (macro_file == NIL || inmacro != NIL)
    {
      scbell ();
      return;
    }
  fd = open (macro_file, O_RDONLY);
  if (fd < 0)
    {
      scbell ();
      return;
    }
  tread (j);
  if (j != macro_magic)
    {
      scbell ();
      goto ret;
    }
  tread (j);
  if (j != MACNOMAX)
    {
      scbell ();
      goto ret;
    }
  tread (j);
  if (j > MACSIZMAX)
    {
      scbell ();
      goto ret;
    }
  tread (macslen);
  if (macslen)
    {
      for (i = MACNOMAX, pm = macros; i; pm++, i--)
	{
	  tread (pm->mbeg);
	  tread (pm->mlen);
	  pm->mstat = MS_NONE;
	}
      read (fd, macbuf, macslen * sizeof (kchar));
    }
  else
    {
      for (i = MACNOMAX, pm = macros; i; pm++, i--)
	{
	  pm->mbeg = 0;
	  pm->mlen = 0;
	  pm->mstat = MS_NONE;
	}
    }
ret:
  close (fd);
}

#undef twrite
#undef tread
