#include "kbxdecterm.h"
#include "cmn.h"

#define MAXTABS 50
#define MAXLENTAB 200
#define FILE_NAME "xdecterm.kb"

short origin_table[] = {

CTRL_A          , 0001,                  -1,
CTRL_B          , 0002,                  -1,
CTRL_C          , 0003,                  -1,
CTRL_D          , 0004,                  -1,
CTRL_E          , 0005,                  -1,
CTRL_F          , 0006,                  -1,
CTRL_G          , 0007,                  -1,
CTRL_H          , 0010,                  -1,
CTRL_I          , 0011,                  -1,
CTRL_J          , 0012,                  -1,
CTRL_K          , 0013,                  -1,
CTRL_L          , 0014,                  -1,
CTRL_M          , 0015,                  -1,
CTRL_N          , 0016,                  -1,
CTRL_O          , 0017,                  -1,
CTRL_P          , 0020,                  -1,
CTRL_Q          , 0021,                  -1,
CTRL_R          , 0022,                  -1,
CTRL_S          , 0023,                  -1,
CTRL_T          , 0024,                  -1,
CTRL_U          , 0025,                  -1,
CTRL_V          , 0026,                  -1,
CTRL_W          , 0027,                  -1,
CTRL_X          , 0030,                  -1,
CTRL_Y          , 0031,                  -1,
CTRL_Z          , 0032,                  -1,

ZB              , 0377,                  -1,
ZB              , 01010,                 -1,
CTRL_ZB         , 0237,                  -1,

Tab             , 01011,                 -1,
CTRL_Tab        , 03011,                 -1,
SHIFT_Tab       , 01411,                 -1,
CTRL_SHIFT_Tab  , 03411,                 -1,
Return          , 01015,                 -1,
CTRL_Return     , 03015,                 -1,
SHIFT_Return    , 01415,                 -1,
CTRL_SHIFT_Return, 03415,                 -1,

N0              , 01260,                 -1,
N0              , 01236,                 -1,
N1              , 01261,                 -1,
N1              , 01234,                 -1,
N2              , 01262,                 -1,
N2              , 01231,                 -1,
N3              , 01263,                 -1,
N3              , 01233,                 -1,
N4              , 01264,                 -1,
N5              , 01265,                 -1,
N6              , 01266,                 -1,
N6              , 01230,                 -1,
N7              , 01267,                 -1,
N7              , 01225,                 -1,
N8              , 01270,                 -1,
N8              , 01227,                 -1,
N9              , 01271,                 -1,
N9              , 01232,                 -1,
COMMA           , 01254,                 -1,
COMMA           , 01253,                 -1,
DOT             , 01256,                 -1,
DOT             , 01237,                 -1,
MINUS           , 01255,                 -1,
Enter           , 01215,                 -1,
UP              , 01122,                 -1,
CTRL_UP         , 03122,                 -1,
SHIFT_UP        , 01522,                 -1,
CTRL_SHIFT_UP   , 03522,                 -1,
DOWN            , 01124,                 -1,
CTRL_DOWN       , 03124,                 -1,
SHIFT_DOWN      , 01524,                 -1,
CTRL_SHIFT_DOWN , 03524,                 -1,
RIGHT           , 01123,                 -1,
CTRL_RIGHT      , 03123,                 -1,
SHIFT_RIGHT     , 01523,                 -1,
CTRL_SHIFT_RIGHT, 03523,                 -1,
LEFT            , 01121,                 -1,
CTRL_LEFT       , 03121,                 -1,
SHIFT_LEFT      , 01521,                 -1,
CTRL_SHIFT_LEFT , 03521,                 -1,
PF1             , 01221,                 -1,
PF2             , 01222,                 -1,
PF2             , 01257,                 -1,
PF3             , 01223,                 -1,
PF3             , 01252,                 -1,
PF4             , 01224,                 -1,
F1              , 01276,                 -1,
CTRL_F1         , 03276,                 -1,
SHIFT_F1        , 01676,                 -1,
CTRL_SHIFT_F1   , 03676,                 -1,
F2              , 01277,                 -1,
CTRL_F2         , 03277,                 -1,
SHIFT_F2        , 01677,                 -1,
CTRL_SHIFT_F2   , 03677,                 -1,
F3              , 01300,                 -1,
CTRL_F3         , 03300,                 -1,
SHIFT_F3        , 01700,                 -1,
CTRL_SHIFT_F3   , 03700,                 -1,
F4              , 01301,                 -1,
CTRL_F4         , 03301,                 -1,
SHIFT_F4        , 01701,                 -1,
CTRL_SHIFT_F4   , 03701,                 -1,
F5              , 01302,                 -1,
CTRL_F5         , 03302,                 -1,
SHIFT_F5        , 01702,                 -1,
CTRL_SHIFT_F5   , 03702,                 -1,
F6              , 01303,                 -1,
CTRL_F6         , 03303,                 -1,
SHIFT_F6        , 01703,                 -1,
CTRL_SHIFT_F6   , 03703,                 -1,
F7              , 01304,                 -1,
CTRL_F7         , 03304,                 -1,
SHIFT_F7        , 01704,                 -1,
CTRL_SHIFT_F7   , 03704,                 -1,
F8              , 01305,                 -1,
CTRL_F8         , 03305,                 -1,
SHIFT_F8        , 01705,                 -1,
CTRL_SHIFT_F8   , 03705,                 -1,
F9              , 01306,                 -1,
CTRL_F9         , 03306,                 -1,
SHIFT_F9        , 01706,                 -1,
CTRL_SHIFT_F9   , 03706,                 -1,
F10             , 01307,                 -1,
CTRL_F10        , 03307,                 -1,
SHIFT_F10       , 01707,                 -1,
CTRL_SHIFT_F10  , 03707,                 -1,
F11             , 01310,                 -1,
CTRL_F11        , 03310,                 -1,
SHIFT_F11       , 01710,                 -1,
CTRL_SHIFT_F11  , 03710,                 -1,
F12             , 01311,                 -1,
CTRL_F12        , 03311,                 -1,
SHIFT_F12       , 01711,                 -1,
CTRL_SHIFT_F12  , 03711,                 -1,
F13             , 01312,                 -1,
CTRL_F13        , 03312,                 -1,
SHIFT_F13       , 01712,                 -1,
CTRL_SHIFT_F13  , 03712,                 -1,
F14             , 01313,                 -1,
CTRL_F14        , 03314,                 -1,
SHIFT_F14       , 01714,                 -1,
CTRL_SHIFT_F14  , 03714,                 -1,
Help            , 01152,                 -1,
CTRL_Help       , 03152,                 -1,
SHIFT_Help      , 01552,                 -1,
CTRL_SHIFT_Help , 03552,                 -1,
Do              , 01147,                 -1,
CTRL_Do         , 03147,                 -1,
SHIFT_Do        , 01547,                 -1,
CTRL_SHIFT_Do   , 03547,                 -1,
F17             , 01316,                 -1,
CTRL_F17        , 03316,                 -1,
SHIFT_F17       , 01716,                 -1,
CTRL_SHIFT_F17  , 03716,                 -1,
F18             , 01317,                 -1,
CTRL_F18        , 03317,                 -1,
SHIFT_F18       , 01717,                 -1,
CTRL_SHIFT_F18  , 03717,                 -1,
F19             , 01320,                 -1,
CTRL_F19        , 03320,                 -1,
SHIFT_F19       , 01720,                 -1,
CTRL_SHIFT_F19  , 03720,                 -1,
F20             , 01321,                 -1,
CTRL_F20        , 03321,                 -1,
SHIFT_F20       , 01721,                 -1,
CTRL_SHIFT_F20  , 03721,                 -1,
Find            , 01150,                 -1,
Find            , 01120,                 -1,
CTRL_Find       , 03150,                 -1,
SHIFT_Find      , 01550,                 -1,
CTRL_SHIFT_Find , 03550,                 -1,
Insert          , 01143,                 -1,
Insert          , 01243,                 -1,
CTRL_Insert     , 03143,                 -1,
SHIFT_Insert    , 01543,                 -1,
CTRL_SHIFT_Insert, 03543,                 -1,
Remove          , 01000,                 -1,
CTRL_Remove     , 03000,                 -1,
SHIFT_Remove    , 01400,                 -1,
CTRL_SHIFT_Remove, 03400,                 -1,
Select          , 01140,                 -1,
Select          , 01127,                 -1,
CTRL_Select     , 03140,                 -1,
SHIFT_Select    , 01540,                 -1,
CTRL_SHIFT_Select, 03540,                 -1,
PrevScreen      , 01125,                 -1,
CTRL_PrevScreen , 03125,                 -1,
SHIFT_PrevScreen, 01525,                 -1,
CTRL_SHIFT_PrevScreen, 03525,                 -1,
NextScreen      , 01126,                 -1,
CTRL_NextScreen , 03126,                 -1,
SHIFT_NextScreen, 01526,                 -1,
CTRL_SHIFT_NextScreen, 03526,                 -1,
TE_FOCUS   ,TE_FOCUS,          -1,
TE_SETXY   ,TE_SETXY,          -1,
LE_TAB     ,LE_TAB,            -1,
TE_CR      ,TE_CR,             -1,
-1};

short special_table[] = {
ESC             , 033,
-1};

#include "gentab.c"

