/*
 *     nanoMir
 *
 *     "tx.c"           Texts
 */

#include "nm.h"
#include "fs.h"
#include "dq.h"
#include "tx.h"
#include "ud.h"
#include "wd.h"
#include "ac.h"
#include "cp.h"
txt texts[MAXFILES];
char *txbuf;
char *afbuf;
tchar *tcbuf;
void (*txILDL) (txt * t, small n);	/* Hook /  */

/* Replace all spaces before tabulation position by tabulation. */
bool TX_ReplaceSpacesByTabs = 0;
/* Tabulation size */
small TX_TabSize = 8;

/* Align N to boundary of A (power of 2) */
#define ALIGN(N,A) (((N) + (A) - 1) & ~((A) - 1))

void
txInit ()
{
  register txt *t;
  register small i;
  for (i = MAXFILES, t = texts; i; t++, i--)
    t->txstat = 0;
  txbuf = GetMain (MAXLUP + 2);
  afbuf = GetMain (MAXLUP);
  tcbuf = (tchar *) GetMain (MAXLPAC * sizeof (tchar));
  txILDL = (void (*)(txt *, small)) br14;
}

txt *
TxNew (bool qundo)
{
  register txt *t;
  register small i;
  for (i = MAXFILES, t = texts; i--; t++)
    if (t->txstat == 0)
      {
#if 0
	if ((t->txustk = DqNew2 (DT_ASC, 0, STKEXT, STKEXT, 0)) == NIL)
	  return NIL;
	if ((t->txdstk = t->txustk->dtwin) == NIL)
	  return NIL;
#else
	if ((t->txustk = DqNew (DT_ASC, 0, STKEXT)) == NIL)
	  return NIL;
	if ((t->txdstk = DqNew (DT_ASC, STKEXT, 0)) == NIL)
	  return NIL;
#endif
	t->txudeq = (qundo ? DqNew (DT_BIN, 0, UNDOEXT) : NIL);
	t->txudcptr = 0;
	t->txudlptr = 0;
	t->txy = 0;
	t->txmarkx = 0;
	t->txmarky = 0;
	t->txlructr = 0;
	t->txwndptr = NIL;
	t->txstat |= TS_BUSY;
	t->txrm = MAXLPAC;
	t->cx = 0;
	t->cy = 0;
	t->tcx = 0;
	t->tcy = 0;
	return (t);
      }
  return (NIL);
}

void
TxDel (register txt * t)
{
  register deq *d;
  DqDel (t->txustk);
  DqDel (t->txdstk);
  if ((d = t->txudeq) != NIL)
    DqDel (d);
  t->txstat = 0;
}

bool
qTxUp (register txt * t)
{
  return (qDqEmpt (t->txustk));
}

bool
qTxDown (register txt * t)
{
  return (qDqEmpt (t->txdstk));
}

void
TxUp (register txt * t)
{
  register small l;
  assert (!qTxUp (t), "TxUp");
  l = DqGetE (t->txustk, txbuf);
  DqAddB (t->txdstk, txbuf, l);
  t->txy--;
}

void
TxDown (register txt * t)
{
  register small l;
  assert (!qTxDown (t), "TxDown");
  l = DqGetB (t->txdstk, txbuf);
  DqAddE (t->txustk, txbuf, l);
  t->txy++;
}

bool
TxSetY (register txt * t, register large y)
{
  while (y != t->txy)
    {
      if (y > t->txy)
	{
	  if (qTxDown (t))
	    return (FALSE);
	  TxDown (t);
	}
      else
	{
	  if (qTxUp (t))
	    return (FALSE);
	  TxUp (t);
	}
    }
  return (TRUE);
}

small
TxRead (register txt * t, char *p)
{
  assert (!qTxDown (t), "TxRead");
  return (DqCopB (t->txdstk, p));
}

void
TxDL (register txt * t)
{
  register small l;
  assert (!qTxDown (t), "TxDl");
  l = DqGetB (t->txdstk, txbuf);
  tundo1add (t, UT_DL, txbuf, l);
  if (t->txmarky > t->txy)
    t->txmarky--;
  wndop (TW_DL, t);
  (*txILDL) (t, -1);
  if (t != LCtxt && t != Btxt)
    stCLines (--(t->flines));
}

void
TxIL (register txt * t, char *a, small l)
{
  DqAddB (t->txdstk, a, l);
  tundo1add (t, UT_IL, a, l);
  if (t->txmarky >= t->txy)
    t->txmarky++;
  wndop (TW_IL, t);
  (*txILDL) (t, 1);
  if (t != LCtxt && t != Btxt)
    stCLines (++(t->flines));
}

void
TxRep (register txt * t, char *a, small l)
{
  register char *p;
  register small lo;
  assert (!qTxDown (t), "TxRep");
  lo = DqGetB (t->txdstk, txbuf);
  tundo2add (t, txbuf, lo, a, l);
  DqAddB (t->txdstk, a, l);
  wndop (TW_RP, t);
}

void
TxEmpt (register txt * t)
{
  register deq *d;
  DqEmpt (t->txdstk);
  DqEmpt (t->txustk);
  if (d = t->txudeq)
    DqEmpt (d);
  t->txudcptr = 0;
  t->txudlptr = 0;
  t->txmarkx = 0;
  t->txmarky = 0;
  t->txy = 0;
  wndop (TW_EM, t);
}

extern large Ly;
extern small Lleng;
extern wnd *Lwnd;
extern tchar *Lebuf;
tchar *
TxInfo (register wnd * w, small y, small * pl)
{
  register txt *t;
  register tchar *tc;
  register small l;
  t = w->wtext;
  if (!TxSetY (t, w->wty + y))
    goto retnil;
  if (w == Lwnd && w->wty + y == Ly)
    {
      tc = Lebuf;
      l = Lleng;
    }
  else
    {
      tc = tcbuf;
      l = TxTRead (t, tc);
    }
  tc += w->wtx;
  if ((l -= w->wtx) < 0)
    {
    retnil:
      l = 0;
      tc = NIL;
    }
  *pl = l;
  return (tc);
}

#ifdef RUSSIAN
#define END_OF_TEXT "[*   *]  : Ctrl+?  : "
#define lendoftext 42
#endif
#ifdef ENGLISH
#define END_OF_TEXT "[* end of text *]  Help: Ctrl+?  File: "
#define lendoftext 39
#endif
small
TxTRead (register txt * t, tchar * tcp)
{
  register small ltc;
  register small laf;
  if (qTxDown (t))
    {
      *scpy (t->finame, scpy (END_OF_TEXT, txbuf)) = 0;
      laf = lendoftext + t->fiddl + t->finml;
      ltc = aftotc (txbuf, laf, tcp);
    }
  else
    {
      laf = TxRead (t, txbuf);
      ltc = aftotc (txbuf, laf, tcp);
    }
  return (ltc);
}

void
TxTIL (txt * t, tchar * tp, small tl)
{
  register small al;
  al = tctoaf (tp, tl, afbuf);
  TxIL (t, afbuf, al);
}

void
TxTRep (txt * t, tchar * tp, small tl)
{
  register small al;
  al = tctoaf (tp, tl, afbuf);
  TxRep (t, afbuf, al);
}

/* Convert Ascii file string to tchar */
small
aftotc (register char *paf, register small laf, register tchar * ptc)
{
  register small ltc;
  register char c;
  register tchar attr;
  for (attr = 0, ltc = 0; laf; laf--)
    {
      switch (c = *paf++)
	{
	case TAB:
	  while (ltc < MAXLPAC)
	    {
	      *ptc++ = (tchar) ' ';
	      /* if (((++ltc) & 07) == 0) */
	      if (((++ltc) % TX_TabSize) == 0)
		break;
	    }
	  break;
	case ACPR:
	  if (laf == 0)
	    goto exit;
	  attr = (tchar) (((*paf++) & ~0200) << 8);
	  laf--;
	  break;
	default:
	  if (ltc == MAXLPAC)
	    goto exit;
	  *ptc++ = ctotc (c) | attr;
	  ltc++;
	  break;
	}
    }
exit:
  return (ltc);
}

/* Convert tchar string to ascii file string */
small
tctoaf (register tchar * ptc, register small ltc, register char *paf)
{
  register small laf;
  register tchar c;
  register char attr;

  if (TX_ReplaceSpacesByTabs)
    {
      small spaces;
      small count;
      small real_len;

      for (attr = 0, laf = 0, spaces = 0, count = 0, real_len = 0; ltc; ltc--, ptc++)
	{
	  if ((c = (char) ((*ptc) >> 8)) != attr)
	    {
	      if (laf >= (MAXLUP - 2))
		goto exit;
	      *paf++ = ACPR;
	      *paf++ = (attr = c) | 0200;
	      laf += 2;
	      real_len += 2;
	    }
	  if (laf == MAXLUP)
	    goto exit;

	  if ((char) (*ptc) == ' ')
	    {
	      if (spaces == 0)
		{
		  /* First version - only for power of 2 tabulation size */
		  /* count = ALIGN (real_len + 1, 8) - real_len; */
		  count = TX_TabSize - (real_len + TX_TabSize) % TX_TabSize;
		}
	      spaces++;
	      if (spaces == count)
		{
		  /*         */
		  if (count != 1)
		    *paf++ = '\t';
		  else
		    *paf++ = ' ';
		  laf++;
		  real_len += count;
		  spaces = 0;
		  count = 0;
		}
	    }
	  else
	    {
	      if (spaces)
		{
		  do
		    {
		      *paf++ = ' ';
		      laf++;
		      real_len++;
		    }
		  while (--spaces && laf < MAXLUP);
		  count = 0;
		  if (laf == MAXLUP)
		    goto exit;
		}

	      *paf++ = (char) (*ptc);
	      laf++;
	      real_len++;
	    }
	}
    }
  else /* !TX_ReplaceSpacesByTabs */
    {
      for (attr = 0, laf = 0; ltc; ltc--)
	{
	  if ((c = (char) ((*ptc) >> 8)) != attr)
	    {
	      if (laf >= (MAXLUP - 2))
		goto exit;
	      *paf++ = ACPR;
	      *paf++ = (attr = c) | 0200;
	      laf += 2;
	    }
	  if (laf == MAXLUP)
	    goto exit;
	  *paf++ = (char) (*ptc++);
	  laf++;
	}
    } /* ! TX_ReplaceSpacesByTabs */

exit:
  return (laf);
}

large
TxLines (txt * t)
{
  large l0, l1;
  if (t == NIL)
    return 0;
  if (t->txustk == NIL)
    l0 = 0;
  else
    l0 = t->txy;
  if (t->txdstk == NIL)
    l1 = 0;
  else
    l1 = DqLines (t->txdstk);
  return l0 + l1;
}
